
__all__ = ['EXP_COEFF_DICT']

from .base import ArithmeticHead

from ..core import init_module, Pair, Expr
init_module.import_heads()
init_module.import_numbers()
init_module.import_lowlevel_operations()

@init_module
def _init(module):
    from ..arithmetic.number_theory import multinomial_coefficients
    module.multinomial_coefficients = multinomial_coefficients

class ExpCoeffDict(ArithmeticHead):
    """
    """

    def is_data_ok(self, cls, data):
        if type(data) is not Pair:
            return 'data must be Pair instance but got %r' % (type(data).__name__)
        variables, exps_coeff_dict = data.pair
        if type(variables) is not tuple:
            return 'data[0] must be tuple but got %r' % (type(variables).__name__)
        if type(exps_coeff_dict) is not dict:
            return 'data[1] must be dict but got %r' % (type(exps_coeff_dict).__name__)
        for exps, coeff in exps_coeff_dict.items():
            if not coeff:
                return 'data[1] contains exp-zero-coeff pair: (%s, %s)' % (exps, coeff)
            if type(exps) is not IntegerList:
                del exps_coeff_dict[exps]
                exps = IntegerList(exps)
                exps_coeff_dict[exps] = coeff
                return 'data[1] keys must be IntegerList instances but got %r' % (type(exps).__name__)
        
    def __repr__(self): return 'EXP_COEFF_DICT'

    def data_to_str_and_precedence(self, cls, data):
        variables, exp_coeff_dict = data
        terms = []
        for exps in sorted(exp_coeff_dict, reverse=True):
            coeff = exp_coeff_dict[exps]
            if type(exps) is not IntegerList:
                # temporary hook for SPARSE_POLY head, remove if block when SPARSE_POLY is gone
                exps = IntegerList(exps)
            factors = []
            for var,exp in zip(variables, exps):
                if not isinstance(var, cls):
                    var = cls(SYMBOL, var)
                factors.append(cls(POW,(var, exp)))
            terms.append(cls(TERM_COEFF, (cls(MUL,factors), coeff)))
        return ADD.data_to_str_and_precedence(cls, terms)

    def to_lowlevel(self, cls, data, pair):
        variables, exp_coeff_dict = data.pair
        n = len(exp_coeff_dict)
        if n==0:
            return 0
        if n==1:
            exps, coeff = dict_get_item(exp_coeff_dict)
            if type(exps) is not IntegerList:
                # temporary hook for SPARSE_POLY head, remove if block when SPARSE_POLY is gone
                exps = IntegerList(exps)
            factors = []
            for var, exp in zip(variables, exps.data):
                if exp==0:
                    continue
                if not isinstance(var, cls):
                    var = cls(SYMBOL, var)
                if exp==1:
                    factors.append(var)
                else:
                    factors.append(cls(POW, (var, exp)))
            if not factors:
                return coeff
            term = MUL.new(cls, factors)
            return term_coeff_new(cls, (term, coeff))
        return pair

    def combine_variables(self, *seq):
        """
        Return a tuple of sorted variables combining given variables.
        """
        variables = set([])
        seq = list(seq)
        while seq:
            s = seq.pop(0)
            if isinstance(s, Expr):
                if s.head is SYMBOL:
                    variables.add(s.data)
                else:
                    variables.add(s)
            elif isinstance(s, str):
                variables.add(s)
            elif isinstance(s, (tuple, list)):
                seq.extend(s)
            elif s is None:
                pass
            else:
                raise TypeError('expected an expression or a sequence of expressions but got %n' % (type(s)))
        return tuple(sorted(variables))

    def make_exponent(self, expr, variables):
        """
        Return exponent list such that expr == variables ** exp_list.
        """
        if expr is 0:
            # shortcut to return exponent of a constant expr
            return IntegerList([0]*len(variables))
        i = list(variables).index(expr)
        if i==-1:
            exp = [0] * len(variables)
        else:
            exp = [0] * (i) + [1] + [0] * (len(variables)-i-1)
        return IntegerList(exp)

    def to_TERM_COEFF_DICT(self, Algebra, data, expr):
        variables, exps_coeff_dict = data
        d = {}
        for exps, coeff in exps_coeff_dict.iteritems():
            d1 = {}
            for exp, var in zip(exps, variables):
                if exp:
                    var = Algebra(var)
                    d1[var] = exp
            term = base_exp_dict_new(Algebra, d1)
            term_coeff_dict_add_item(Algebra, d, term, 1)
        return term_coeff_dict_new(Algebra, d)
        
    def to_EXP_COEFF_DICT(self, cls, data, expr, variables = None):
        if variables is None:
            return expr
        evars, edata = data.pair
        if evars==variables:
            return expr
        variables = self.combine_variables(evars, variables)
        levars = list(evars)
        l = []
        for v in variables:
            try:
                i = levars.index(v)
            except ValueError:
                i = None
            l.append(i)
        d = {}
        for exps, coeff in edata.iteritems():
            new_exps = IntegerList([(exps[i] if i is not None else 0) for i in l])
            d[new_exps] = coeff
        return cls(self, Pair(variables, d))

    def neg(self, cls, expr):
        return self.commutative_mul_number(cls, expr, -1)

    def add_number(self, cls, lhs, rhs, inplace=False):
        if not rhs:
            return lhs
        lvars, ldict = lhs.data.pair
        zero_exp = self.make_exponent(0, lvars)
        if inplace and lhs.is_writable:
            dict_add_item(cls, ldict, zero_exp, rhs)
            return lhs
        d = ldict.copy()
        dict_add_item(cls, d, zero_exp, rhs)
        return cls(self, Pair(lvars, d))

    def add(self, cls, lhs, rhs, inplace=False):
        lvars, ldict = lhs.data.pair
        rhead, rdata = rhs.pair
        if rhead is not EXP_COEFF_DICT:
            rhs = rhead.to_EXP_COEFF_DICT(cls, rdata, rhs, lvars)
            rhead, rdata = rhs.pair
        rvars, rdict = rdata.pair
        if lvars == rvars:
            if inplace and lhs.is_writable:
                dict_add_dict(cls, ldict, rdict)
                return lhs
            d = ldict.copy()
            dict_add_dict(cls, d, rdict)
            return cls(self, Pair(lvars, d))
        variables = tuple(sorted(set(lvars + rvars)))
        lhs = self.to_EXP_COEFF_DICT(cls, lhs.data, lhs, variables)
        rhs = self.to_EXP_COEFF_DICT(cls, rhs.data, rhs, variables)
        d = lhs.data.data.copy()
        dict_add_dict(cls, d, rhs.data.data)
        return cls(self, Pair(variables, d))

    def sub(self, cls, lhs, rhs):
        return self.add(cls, lhs, -rhs)

    def commutative_mul(self, cls, lhs, rhs):
        rhead, rdata = rhs.pair
        if rhead is NUMBER:
            return self.commutative_mul_number(cls, lhs, rdata)
        lvars, ldict = lhs.data.pair
        if rhead is not EXP_COEFF_DICT:
            rhs = rhead.to_EXP_COEFF_DICT(cls, rdata, rhs, lvars)
            rhead, rdata = rhs.pair
        if rhead is EXP_COEFF_DICT:
            rvars, rdict = rdata.pair
            d = {}
            if lvars == rvars:
                exp_coeff_dict_mul_dict(cls, d, ldict, rdict)
                return cls(self, Pair(lvars, d))
            variables = tuple(sorted(set(lvars + rvars)))
            lhs = self.to_EXP_COEFF_DICT(cls, lhs.data, lhs, variables)
            rhs = self.to_EXP_COEFF_DICT(cls, rhs.data, rhs, variables)
            exp_coeff_dict_mul_dict(cls, d, lhs.data[1], rhs.data[1])
            return cls(self, Pair(variables, d))
        raise NotImplementedError(`self, rhs.head`)

    def commutative_mul_number(self, cls, lhs, rhs):
        lvars, ldict = lhs.data.pair
        if rhs==0:
            return cls(self, Pair(lvars, {}))
        if rhs==1:
            return lhs
        d = {}
        for exps, coeff in ldict.iteritems():
            d[exps] = coeff * rhs
        return cls(self, Pair(lvars, d))

    non_commutative_mul_number = commutative_mul_number

    def combine_ncmul_exponents(self, lexps, rexps, variables):
        """
        Return exponents of non-commutative multiplication:
          variables ** ([lexps] + [rexps]) -> variables ** exps
        TODO: move the algorithm to expr.py and implement its C version.
        """
        exps = list(lexps) + list(rexps)
        n = len(exps)
        i0 = 0
        while 1:
            i = i0
            while i < n and not exps[i]:
                i += 1
            if i==n:
                break
            j = i+1
            while j < n and not exps[j]:
                j += 1
            if j==n:
                break
            if variables[i] == variables[j]:
                exps[i] += exps[j]
                exps[j] = 0
                i0 = 0
            else:
                i0 = j
        return IntegerList(exps)

    def eliminate_trivial_exponents(self, Algebra, exps_coeff_dict, variables):
        """
        Eliminate common trivial exponents (=0) from the set of exponents
        and return the corresponding non-trivial list of variables.
        Note: exps_coeff_dict will be changed in place.
        """
        has_non_zeros = [0] * len(variables)
        for exps in exps_coeff_dict:
            n = 0
            for i, exp in enumerate(exps):
                if has_non_zeros[i]:
                    n += 1
                    continue
                if exp:
                    n += 1
                    has_non_zeros[i] = 1
            if n==len(variables): # no common trivial exponents
                return variables
        non_trivial_indices = [i for i in range(len(variables)) if has_non_zeros[i]]
        assert len(non_trivial_indices) < len(variables),`non_trivial_indices, variables`
        for exps in exps_coeff_dict.keys():
            coeff = exps_coeff_dict[exps]
            del exps_coeff_dict[exps]
            exps = IntegerList([exps[i] for i in non_trivial_indices])
            dict_add_item(Algebra, exps_coeff_dict, exps, coeff)
        return tuple([variables[i] for i in non_trivial_indices])
        
    def non_commutative_mul(self, cls, lhs, rhs):
        rhead, rdata = rhs.pair
        if rhead is NUMBER:
            return self.non_commutative_mul_number(cls, lhs, rdata)
        lvars, ldict = lhs.data.pair
        if rhead is not EXP_COEFF_DICT:
            rhs = rhead.to_EXP_COEFF_DICT(cls, rdata, rhs, lvars)
            rhead, rdata = rhs.pair
        if rhead is EXP_COEFF_DICT:
            rvars, rdict = rdata.pair
            variables = lvars + rvars
            lhs = self.to_EXP_COEFF_DICT(cls, lhs.data, lhs, variables)
            rhs = self.to_EXP_COEFF_DICT(cls, rhs.data, rhs, variables)
            d = {}
            # TODO: move the following double-loop to expr.py and implement it in C.
            for lexp, lcoeff in ldict.iteritems():
                for rexp, rcoeff in rdict.iteritems():
                    exp = self.combine_ncmul_exponents(lexp, rexp, variables)
                    dict_add_item(cls, d, exp, lcoeff * rcoeff)
            variables = self.eliminate_trivial_exponents(cls, d, variables)
            return cls(self, Pair(variables, d))
        raise NotImplementedError(`self, rhs.head`)
    
    def commutative_div(self, cls, lhs, rhs):
        return lhs * rhs ** -1
    
    def pow(self, cls, base, exp):
        variables, exp_coeff_dict = base.data.pair
        if isinstance(exp, Expr) and exp.head is NUMBER and isinstance(exp.data, inttypes):
            exp = exp.data
        if isinstance(exp, inttypes):
            return self.pow_number(cls, base, exp)
        expr = cls(POW, (base, exp))
        variables = self.combine_variables(variables, expr)
        exps = self.make_exponent(expr, variables)
        return cls(self, Pair(variables, {exps:1}))

    def pow_number(self, cls, base, exp):
        variables, exp_coeff_dict = base.data.pair
        if isinstance(exp, inttypes):
            if exp==0:
                return cls(self, Pair(variables, {(0,)*len(variables):1}))
            if exp==1:
                return base
            if exp>1:
                exps_coeff_list = base.data.data.items()
                m = len(variables)
                mdata = multinomial_coefficients(len(exps_coeff_list), exp)
                d = {}
                for e,c in mdata.iteritems():
                    new_exps = IntegerList([0]*m)
                    new_coeff = c
                    for e_i, (exps,coeff) in zip(e, exps_coeff_list):
                        new_exps += exps * e_i
                        new_coeff *= coeff ** e_i
                    dict_add_item(cls, d, new_exps, new_coeff)
                return cls(self, Pair(variables, d))
            # exp is negative integer
            if len(exp_coeff_dict)==1:
                exps, coeff = dict_get_item(exp_coeff_dict)
                inv_coeff =  number_div(cls, 1, coeff)
                return cls(self, Pair(variables, {-exps: inv_coeff}))
        expr = cls(POW, (base, exp))
        variables = self.combine_variables(variables, expr)
        exps = self.make_exponent(expr, variables)
        return cls(self, Pair(variables, {exps:1}))

EXP_COEFF_DICT = ExpCoeffDict()



import pandas
import numpy
import matplotlib
from matplotlib import pyplot as plt
from pymaclab import newFAVAR
import os
import copy
import datetime as dt
from scikits import timeseries as ts
from scikits.timeseries.lib.interpolate import interp_masked1d,backward_fill
from pymaclab.dattrans import fred
import glob
fred_key = 'c5c6ec1dd5e1eb81b2727053690986db'
fred.key(fred_key)

modconfig = {}
# This model's name or title, used for pickling results
modname = 'BLOOM_favar_model'
modconfig['modname'] = modname
# Get the data from the directory, later we will add the St. Louis Fed
datafile = 'October_Updated_finaldata.csv'
modconfig['datafile'] = datafile
datapath = '../data/'
modconfig['datapath'] = datapath
data = pandas.read_csv(os.path.join(datapath,datafile),delimiter=';')
data = pandas.DataFrame(numpy.array([data['uncert'].values,data['spindex'].values,data['ffr'].values,
                                     data['employment'].values,data['ip'].values,data['copper'].values]).T,
                        columns=['UNCERT','SPINDEX','FFR','EMP','IP','COPPER'])
columns = list(data.columns)
# Name list for St. Louis Fed data to be appended
stfedli = ['GS2','GS10','PPIFGS','CPIAUCSL','CPILFESL','M2SL','CFNAI','IPDCONGD',
           'TCU','MCUMFN','HOUST','NAPM','USSLIND','IC4WSA','PSAVERT','MZM','TB3MS',
           'TB6MS','GS1','GS5','UMCSENT','LNS12032195','AAA','EMRATIO','OILPRICE',
           'DSPIC96','MORTG','CURRENCY','BAA','PCE','TOTALSL','GPDI','NAPMII',
           'NAPMNOI','NAPMEI','NAPMSDI','NAPMPRI','NAPMPI','CPIUFDNS','CUSR0000SAC',
           'CUSR0000SEHF','CUSR0000SAS4','PPIITM','GS7','GS3','PCEPI','PCEPILFE',
           'PCEDG','PCEND','PCES','NONREVSL','TWEXMMTH','MZMV','M2V','M1V']

for elem in stfedli:
    columns.append(elem)
# Define frequency of data
freq = 'M'
modconfig['freq'] = freq
###################################################################################################################
######################## Define here the lags for the data and the factors ########################################
### There is a trade-off here, you can chose not to use any lags on the data itself and let the factors capture ###
### all of the data's persistence, in that case your lag structure on the factors needs to be long enough #########
# Should the original data be filtered using their own lags?
xdata_filter = False
modconfig['xdata_filter'] = xdata_filter
# Define the number of lags in the VAR, only applicable if option above was chosen to be true
nlags = 6
modconfig['nlags'] = nlags
# Define the number of lags in the VAR of the factors, we always keep this at 1, becaus we are using a companion structure
flags = 6
modconfig['flags'] = flags
# Define the number of common static factors to be employed, but this can later be overriden by automatic selection
sfacs = 12
modconfig['sfacs'] = sfacs
###################################################################################################################
###################################################################################################################
# Define the convergence criterion in PERCENTAGES
conv_crit = 0.001
modconfig['conv_crit'] = conv_crit
# Should IRFs and confidence intervals be computed?
do_irfs = True
modconfig['do_irfs'] = do_irfs
# Should all directories be deleted before a model run?
predel = True
modconfig['predel'] = predel
# How many periods for IRFs ?
irfp = 48
modconfig['irfp'] = irfp
# Calculate variance decomposition?
do_vdc = True
modconfig['do_vdc'] = do_vdc
# How many periods forecast for variance decomposition ?
vdcp = 60
modconfig['vdcp'] = vdcp
# Graph only IR with respect to identified variable
irf_ident_graph = True
modconfig['irf_ident_graph'] = irf_ident_graph
# How many periods for the XDATA IRFs ?
xirfp = 40
modconfig['xirfp'] = xirfp
# Use the svname matrix for alternative shocks?
use_svnames = False
modconfig['use_svnames'] = use_svnames
# Rescale the IRFs ?
rescale = True
modconfig['rescale'] = rescale
# Transform log variables in IRFs into percentages (only useful with rescaling) ?
translog = True
modconfig['translog'] = translog
# Transform the percentage responses back into absolutes from the mean ?
transperc = False
modconfig['transperc'] = transperc
# Add impact matrix to Cholesky responses ?
cholimp = False
modconfig['cholimp'] = cholimp
# How many matrices for MA Representation, this has to be a bit bigger than the number of time periods in the IRs
maxphi = max(vdcp,irfp,xirfp)+2
modconfig['maxphi'] = maxphi
# Boostrap IRF confidence intervals ?
bootstrap = True
modconfig['bootstrap'] = bootstrap
# Draw a horizontal zero line into each IRF Plot
hzline = True
modconfig['hzline'] = hzline
# Should IRFs be annotated
irf_anot = False
modconfig['irf_anot'] = irf_anot
# How thick should the hline be?
hzline_width = 1.0
modconfig['hzline_width'] = hzline_width
# How many draws for bootstrap ?
bdraw = 1000
modconfig['bdraw'] = bdraw
# Use multicore processors to speed up bootstrap?
multicpu = True
modconfig['multicpu'] = multicpu
# If bootstraps are split into chunks, what is each chunk's size (matters for memory efficiency)
bdraw_chunk = 200
modconfig['bdraw_chunk'] = bdraw_chunk
# Specify number of cpu cores or set to 'auto'
ncpus = 'auto'
modconfig['ncpus'] = ncpus
# Level of significance for bootstrap confidence intervals
signif = 'mix' # implies 1-alpha = 0.66, use 'mix' to graph a variety of CI
modconfig['signif'] = signif
# Boostrap with replacement ?
breplace = False
modconfig['breplace'] = breplace
# What kind of intercept/time trend?
const_term = None # Can be 'None', 'cc' or 'tt'
modconfig['const_term'] = const_term
# Pickle the model run results and settings
pickle_switch = False
modconfig['pickle_switch'] = pickle_switch
########################### use ONE of the identification schemes, not many ###################
# Employ BBE setup of the FAVAR?
use_bbe_ident = True
modconfig['use_bbe_ident'] = use_bbe_ident
# Employ simple Cholesky factorisation of the factor innovation matrix to diagonalize shocks
use_chol_ident = False
modconfig['use_chol_ident'] = use_chol_ident
################################################################################################
####################### This is experimental stuff, need to clarify the difference between the two #####################
# Parametric bootstrap vs. non-parametric bootstrap?
param_bootstrap = False
modconfig['param_bootstrap'] = param_bootstrap
killian_bsinbs = True
modconfig['killian_bsinbs'] = killian_bsinbs
########################################################################################################################
########################################## Graphing options for IRFs and other graphs ##################################
graph_options = {}
graph_options['labels'] = {}
graph_options['labels']['title'] = True
graph_options['labels']['title_top_only'] = True
graph_options['labels']['xlabel'] = False
graph_options['labels']['ylabel'] = True
graph_options['labels']['xlabel_fs'] = 10
graph_options['labels']['ylabel_fs'] = 10
graph_options['labels']['text_fs'] = 10
graph_options['labels']['title_fs'] = 12
graph_options['labels']['label_fs'] = 8
graph_options['labels']['xticks_fs'] = 8
graph_options['labels']['yticks_fs'] = 8
graph_options['labels']['legend_fs'] = 8
graph_options['labels']['use_tex'] = False
graph_options['lines'] = {}
graph_options['lines']['width'] = 2.0
graph_options['grid'] = True
graph_options['irfs'] = {}
graph_options['irfs']['outer_colour'] = '#a4a4a4'
graph_options['irfs']['inner_colour'] = '#c6c6c6'
graph_options['irfs']['line_colour'] = 'black'
graph_options['irfs']['biased_line'] = True # This allows you to also show the biased IRF when Killian's b-i-b option is on
graph_options['irfs']['biased_line_type'] = 'k--'
modconfig['graph_options'] = graph_options
#########################################################################################################################
# Should recessions be shown in the factor graphs?
plot_rec = True
modconfig['plot_rec'] = plot_rec
# Colour and transparency of recession shades
rec_colour = '#a4a4a4'
rec_transp = 0.2
modconfig['rec_colour'] = rec_colour
modconfig['rec_transp'] = rec_transp
##########################################################################################################################
# Options for BBE identification setup
separate_vars = []
separate_vars.append('UNCERT')
modconfig['separate_vars'] = separate_vars
slow_vars = []
slow_vars.append('EMP')
slow_vars.append('IP')
slow_vars.append('IPDCONGD')
slow_vars.append('PPIFGS')
slow_vars.append('TCU')
slow_vars.append('MCUMFN')
slow_vars.append('CPIAUCSL')
slow_vars.append('CPILFESL')
slow_vars.append('HOUST')
slow_vars.append('IC4WSA')
slow_vars.append('PSAVERT')
slow_vars.append('LNS12032195')
slow_vars.append('EMRATIO')
slow_vars.append('DSPIC96')
slow_vars.append('PCE')
slow_vars.append('GPDI')
slow_vars.append('CPIUFDNS')
slow_vars.append('CUSR0000SAC')
slow_vars.append('CUSR0000SEHF')
slow_vars.append('CUSR0000SAS4')
slow_vars.append('PPIITM')
slow_vars.append('PCEPI')
slow_vars.append('PCEPILFE')
slow_vars.append('PCEDG')
slow_vars.append('PCEND')
slow_vars.append('PCES')
slow_vars.append('MZMV')
slow_vars.append('M2V')
slow_vars.append('M1V')
modconfig['slow_vars'] = slow_vars
fast_vars = []
fast_vars.append('COPPER')
fast_vars.append('OILPRICE')
fast_vars.append('CFNAI')
fast_vars.append('NAPM')
fast_vars.append('NAPMII')
fast_vars.append('NAPMNOI')
fast_vars.append('NAPMEI')
fast_vars.append('NAPMSDI')
fast_vars.append('NAPMPRI')
fast_vars.append('NAPMPI')
fast_vars.append('UMCSENT')
fast_vars.append('NONREVSL')
fast_vars.append('SPINDEX')
fast_vars.append('FFR')
fast_vars.append('GS10')
fast_vars.append('GS2')
fast_vars.append('M2SL')
fast_vars.append('MZM')
fast_vars.append('USSLIND')
fast_vars.append('TB3MS')
fast_vars.append('TB6MS')
fast_vars.append('GS1')
fast_vars.append('GS5')
fast_vars.append('AAA')
fast_vars.append('MORTG')
fast_vars.append('CURRENCY')
fast_vars.append('BAA')
fast_vars.append('TOTALSL')
fast_vars.append('GS7')
fast_vars.append('GS3')
fast_vars.append('TWEXMMTH')

modconfig['fast_vars'] = fast_vars
# You can also fix the number of factors for the slow variables here, or let them be estimated using Ng-Bai criterion
slow_facs = sfacs-2
modconfig['slow_facs'] = slow_facs
# Define the names of the vars used
vnames = []
for sname in columns:
    vnames.append(sname)
modconfig['vnames'] = vnames
# Define the alternative shocks
svnames = {}
for sname in columns:
    svnames[sname] = 1.0
svnames['UNCERT'] = 112.0
modconfig['svnames'] = vnames
# Set the transperc_dic
transperc_dic = {}
for name in vnames:
    transperc_dic[name] = False
modconfig['transperc_dic'] = transperc_dic
# Define the names for the vnames used in plotting, at the moment copy the short name
pnames = {}
for sname in columns:
    pnames[sname] = sname
#Define the variables which are cointegrated, here the order also matters !
conames = []
conames.append('INF')
conames.append('LHUR')
conames.append('FYFF')
modconfig['conames'] = conames
# Define the matrix betta for the long-run cointegrating relationship
betta_coint = numpy.array([[1,2,3],[1,2,3],[1,2,3]])
modconfig['betta_coint'] = betta_coint


# Function to transform the raw data according to some classification
# 1 = levels
# 2 = first seasonal difference
# 3 = second seasonal difference
# 4 = log level
# 5 = log first seasonal difference
# 6 = log second seasonal difference
# 7 = detrend log using hp filter monthly data
# 8 = detrend log using hp filter quarterly data
# 16 = log second seasonal difference
# 17 = (1-L)(1-L^12)
# 18 = log of (1-L)*annualizing factor (i.e. x4 for quarterly and x12 for monthly
# 19 = filter the level using simple time-trend
# 20 = filter the log using simple time-trend
# Create the data modification dictionary
tcode_dic = {}
clist = [1,4,1,7,7,4]
stlouislist = [1,1,7,7,7,7,1,7,1,1,4,1,1,4,1,7,1,1,1,1,1,7,1,4,4,
               7,1,7,1,7,7,7,1,1,1,1,1,1,7,7,7,7,7,1,1,7,7,7,7,7,7,4,4,4,4]
clist = clist + stlouislist
for i1,vname in enumerate(vnames):
    tcode_dic[vname] = clist[i1]
modconfig['tcode'] = tcode_dic

# Define a matrix for scaling the orthogonalized shock
svnames = {}
for name in vnames:
    svnames[name] = 1.0
svnames['UNCERT'] = 112.0
modconfig['svnames'] = svnames
# Get the individual series of interest, also take natural logs where wanted
loglist = []
modconfig['loglist'] = loglist
orig_li = vnames[:-len(stfedli)]
tlist = []
for name in orig_li:
    if name in loglist:
        tlist.append(numpy.log(data[name].values))
    else:
        tlist.append(data[name].values)
matd = numpy.array(tlist)
matd = matd[:,:-2]
# Transpose and get the , but first add St. Lous Fed data, also save in temporary file
matd = matd.T
# Also create the table for the data
tmpstr = ''
tmpstr = tmpstr + r"\begin{sidewaystable}[H]\centering"+"\n"
tmpstr = tmpstr + r"\caption{Summary of all data used in FAVAR}"+"\n"
tmpstr = tmpstr + r"\tiny"+"\n"
tmpstr = tmpstr + r"\begin{tabular}{@{}lllllll@{}}\toprule"+"\n"
tmpstr = tmpstr + r"No. & Description & Code & Source & Original Units & Variable Type & TCode\\\midrule"+"\n"
newstr = str(1)+r" & "+r"Bloom et al.'s policy uncertainty index"+r" & "+r"UNCERT"+r" & "+r"Bloom et al."+r" & "+r"Index"+r" & "+r"Identified"+r" & "+str(4)+r"\\"+"\n"
tmpstr = tmpstr + newstr
newstr = str(2)+r" & "+r"Standard and Poor's 500 US Stock Market Index"+r" & "+r"SPINDEX"+r" & "+r"Bloom et al."+r" & "+r"Index"+r" & "+r"Fast"+r" & "+str(4)+r"\\"+"\n"
tmpstr = tmpstr + newstr
newstr = str(3)+r" & "+r"US Federal Funds Rate"+r" & "+r"FFR"+r" & "+r"Bloom et al."+r" & "+r"Percent"+r" & "+r"Fast"+r" & "+str(1)+r"\\"+"\n"
tmpstr = tmpstr + newstr
newstr = str(4)+r" & "+r"US Total Civilian Employment"+r" & "+r"EMP"+r" & "+r"Bloom et al."+r" & "+r"Total no. of persons in 1000s"+r" & "+r"Slow"+r" & "+str(7)+r"\\"+"\n"
tmpstr = tmpstr + newstr
newstr = str(5)+r" & "+r"US Industrial Production Index"+r" & "+r"IP"+r" & "+r"Bloom et al."+r" & "+r"Index"+r" & "+r"Slow"+r" & "+str(7)+r"\\"+"\n"
tmpstr = tmpstr + newstr
newstr = str(6)+r" & "+r"Price of Copper in US dollars"+r" & "+r"COPPER"+r" & "+r"Bloom et al."+r" & "+r"US dollars per tonne"+r" & "+r"Fast"+r" & "+str(4)+r"\\"+"\n"
tmpstr = tmpstr + newstr
for i1,namo in enumerate(stfedli):
    print "Now acquiring data for: "+namo
    resu = fred.observations(namo)
    resuf = fred.series(series_id=namo)
    sera = resuf['seriess']['series']
    if namo in slow_vars: stringo = 'Slow'
    elif namo in fast_vars: stringo = 'Fast'
    elif namo in separate_vars: stringo = 'Identified'
    if namo == 'LNS12032195': stitle = sera['title'].split(',')[0]
    else: stitle = sera['title'].replace(r'&','and')
    newstr = str(i1+7)+r" & "+stitle+r" & "+namo+r" & "+r"St. Louis Fed"+r" & "+ sera['units']+r" & "+stringo+r" & "+str(stlouislist[i1])+r"\\"+"\n"
    tmpstr = tmpstr + newstr
    freqf = resuf['seriess']['series']['frequency_short']
    print "Done..."
    valo = [x['value'] for x in resu['observations']['observation']]
    valo2 = numpy.zeros(len(valo))
    for i1,elem in enumerate(valo):
        if elem == '.': valo2[i1] = float(valo2[i1-1])
        else: valo2[i1] = float(elem)
    valo = numpy.array(valo2)
    dato = [x['date'] for x in resu['observations']['observation']]
    if freqf == 'D':
        tso = ts.time_series(valo,dates=dato,freq='D')
        tso2 = ts.convert(tso,'M',func=numpy.ma.mean)
    elif freqf == 'W':
        tso = ts.time_series(valo,dates=dato,freq='D')
        tso2 = ts.convert(tso,'M',func=numpy.ma.mean)
    elif freqf == 'M':
        tso = ts.time_series(valo,dates=dato,freq='M')
        tso2 = copy.deepcopy(tso)
    elif freqf == 'Q':
        tso = ts.time_series(valo,dates=dato,freq='Q')
        tso2 = ts.convert(tso,'M')
        tso2 = interp_masked1d(tso2)
    valor = tso2['1985-01-01':'2011-09-01']
    dator = valor.dates
    valor = valor.data
    valor = numpy.reshape(valor,(len(valor),1))
    matd = numpy.hstack((matd,valor))

tmpstr = tmpstr + r"\bottomrule"+"\n"
tmpstr = tmpstr + r"\multicolumn{7}{l}{{\bf Notes:} Transformation codes are: 1=levels, 2=first seasonal difference, 3=second seasonal difference, 4=log level, 5=log first seasonal difference, 6=log second seasonal difference,}\\"+"\n"
tmpstr = tmpstr + r"\multicolumn{7}{l}{7=log hp-filtered monthly data. Note that investment data were intrapolated in order to obtain monthly from quarterly data. The choice of fast/slow variables in this table reflects the benchmark ordering.}"+"\n"
tmpstr = tmpstr + r"\end{tabular}"+"\n"
tmpstr = tmpstr + r"\end{sidewaystable}"+"\n"

'''
if 'table1.tex' in os.listdir('../tex_papers/bloom_favar/'):
    os.remove('../tex_papers/bloom_favar/table1.tex')
f1 = open('../tex_papers/bloom_favar/table1.tex','w')
f1.write(tmpstr)
f1.close()
'''

cols = matd.shape[1]
rows = matd.shape[0]
modconfig['cols'] = cols
modconfig['rows'] = rows
# In total there are three US recessions in the sample considered here
rdates = []
rdates.append([ts.Date(freq='M',year=1990,month=7,day=1),ts.Date(freq='M',year=1991,month=3,day=1)])
rdates.append([ts.Date(freq='M',year=2001,month=3,day=1),ts.Date(freq='M',year=2001,month=11,day=1)])
rdates.append([ts.Date(freq='M',year=2007,month=12,day=1),ts.Date(freq='M',year=2009,month=7,day=1)])

# Before running the VAR, plot the uncertainty data for the paper
from matplotlib.ticker import MultipleLocator, FormatStrFormatter
import matplotlib.dates as mdates
majorLocator   = MultipleLocator(1)
majorFormatter = FormatStrFormatter('%d')
minorLocator   = MultipleLocator(1)
years    = mdates.YearLocator()   # every year
months   = mdates.MonthLocator()  # every month
yearsFmt = mdates.DateFormatter('%Y') 
glopt = modconfig['graph_options']['labels']
gridopt = modconfig['graph_options']['grid']        
modconfig['modname']
# Check if directory exists
if modname not in os.listdir('../graphs/'):
    os.mkdir('../graphs/'+modname)
uncert = matd[:,0]
fig1 = plt.figure()
ax1 = fig1.add_subplot(1,1,1)
if gridopt: ax1.grid()
for label in ax1.get_xticklabels():
    label.set_rotation(55)       
if glopt['xlabel']: ax1.set_xlabel('Time',fontsize=glopt['label_fs'])
if glopt['ylabel']: ax1.set_ylabel('Index 100=(1985-2000)',fontsize=glopt['label_fs'])
datli = [x.datetime for x in dator]
ax1.plot(datli[-len(uncert):],uncert,'k-',linewidth=2.0)
dato1 = dt.datetime(1985,12,31)
if glopt['title']: ax1.set_title('Uncertainty Index (Bloom et al.)',fontsize=glopt['title_fs'])
fig1.savefig('../graphs/'+modname+'/'+'UNCERT.eps',bbox_inches='tight')

favar = newFAVAR(data=matd,dates=dator,rdates=rdates,freq=freq,vnames=vnames,pnames=pnames,svnames=svnames,irfs=True,rescale=rescale,boot=True,plot=True,sfacs=sfacs,init=None,conf=modconfig,mesg=True)


# Also create the table for the impulse responses
tmpstr = ''
tmpstr = tmpstr + r"\begin{tabular}{@{}lll}"+"\n"
substr = ''
rowo = 7
colo = 3
rowo_count = 0
switcho = False
globli = glob.glob('../graphs/'+modname+'/shock_irfs/irf_12_*.eps')
globli.sort()
for i1,filo in enumerate(globli):
    if i1 != 0 and float(rowo_count+1)%float(rowo) == 0.0:
        tmpstr = tmpstr + r"\end{tabular}"+"\n"+r"\newpage"+"\n"+r"\begin{tabular}{@{}lll}"+"\n"
        rowo_count += 1
        switcho = True
    if i1 != 0 and float(i1+1)%float(colo) > 0.0: substr = substr + r" & \includegraphics[scale=0.27]{"+"../"+filo[:-4]+"}"
    elif i1 == 0: substr = substr + r" & \includegraphics[scale=0.27]{"+"../"+filo[:-4]+"}"
    else:
        substr = substr + r" & \includegraphics[scale=0.27]{"+"../"+filo[:-4]+"}"
        if i1 != 0: tmpstr = tmpstr + copy.deepcopy(substr[2:] + r"\\" + "\n")
        if not switcho: rowo_count += 1
        substr = ''
    switcho = False
tmpstr = tmpstr + r"\end{tabular}"+"\n"
if 'table2.tex' in os.listdir('../tex_papers/bloom_favar/'):
    os.remove('../tex_papers/bloom_favar/table2.tex')
f1 = open('../tex_papers/bloom_favar/table2.tex','w')
f1.write(tmpstr)
f1.close()

# Also create the table for the impulse responses, copper and oil
tmpstr = ''
tmpstr = tmpstr + r"\begin{tabular}{@{}lll}"+"\n"
substr = ''
rowo = 7
colo = 3
rowo_count = 0
switcho = False
globli = glob.glob('../graphs/'+modname+'/oilcopper/irf_12_*.eps')
globli.sort()
for i1,filo in enumerate(globli):
    if i1 != 0 and float(rowo_count+1)%float(rowo) == 0.0:
        tmpstr = tmpstr + r"\end{tabular}"+"\n"+r"\newpage"+"\n"+r"\begin{tabular}{@{}lll}"+"\n"
        rowo_count += 1
        switcho = True
    if i1 != 0 and float(i1+1)%float(colo) > 0.0: substr = substr + r" & \includegraphics[scale=0.27]{"+"../"+filo[:-4]+"}"
    elif i1 == 0: substr = substr + r" & \includegraphics[scale=0.27]{"+"../"+filo[:-4]+"}"
    else:
        substr = substr + r" & \includegraphics[scale=0.27]{"+"../"+filo[:-4]+"}"
        if i1 != 0: tmpstr = tmpstr + copy.deepcopy(substr[2:] + r"\\" + "\n")
        if not switcho: rowo_count += 1
        substr = ''
    switcho = False
tmpstr = tmpstr + r"\end{tabular}"+"\n"
if 'table3.tex' in os.listdir('../tex_papers/bloom_favar/'):
    os.remove('../tex_papers/bloom_favar/table3.tex')
f1 = open('../tex_papers/bloom_favar/table3.tex','w')
f1.write(tmpstr)
f1.close()

# Also create the table for the impulse responses, copper oil survey
tmpstr = ''
tmpstr = tmpstr + r"\begin{tabular}{@{}lll}"+"\n"
substr = ''
rowo = 7
colo = 3
rowo_count = 0
switcho = False
globli = glob.glob('../graphs/'+modname+'/oilcoppersurvey/irf_12_*.eps')
globli.sort()
for i1,filo in enumerate(globli):
    if i1 != 0 and float(rowo_count+1)%float(rowo) == 0.0:
        tmpstr = tmpstr + r"\end{tabular}"+"\n"+r"\newpage"+"\n"+r"\begin{tabular}{@{}lll}"+"\n"
        rowo_count += 1
        switcho = True
    if i1 != 0 and float(i1+1)%float(colo) > 0.0: substr = substr + r" & \includegraphics[scale=0.27]{"+"../"+filo[:-4]+"}"
    elif i1 == 0: substr = substr + r" & \includegraphics[scale=0.27]{"+"../"+filo[:-4]+"}"
    else:
        substr = substr + r" & \includegraphics[scale=0.27]{"+"../"+filo[:-4]+"}"
        if i1 != 0: tmpstr = tmpstr + copy.deepcopy(substr[2:] + r"\\" + "\n")
        if not switcho: rowo_count += 1
        substr = ''
    switcho = False
tmpstr = tmpstr + r"\end{tabular}"+"\n"
if 'table4.tex' in os.listdir('../tex_papers/bloom_favar/'):
    os.remove('../tex_papers/bloom_favar/table4.tex')
f1 = open('../tex_papers/bloom_favar/table4.tex','w')
f1.write(tmpstr)
f1.close()


# Create commodity markets graph
fig2 = plt.figure()
bnames = ['COPPER','OILPRICE']
bcols = ['#000000','#B5B3B4']
pron = ['min','min']
btype = ['-','-']
indx = []
for namos in bnames:
    indx.append(favar.vnames.index(namos))
ax2 = fig2.add_subplot(1,1,1)
plt.grid()
for i1,elem in enumerate(indx):
    if tcode_dic[bnames[i1]] in [4,5,6,7,8]:
        percen = 100.0
    else:
        percen = 1.0
    ax2.plot(favar.phisc[:49,elem,-1].flatten()*percen,label=bnames[i1],color=bcols[i1],linestyle=btype[i1])
    ax2.set_ylabel('Percentage Deviation',fontsize=glopt['label_fs'])
    '''
    if pron[i1] == 'max':
        val = max(favar.phisc[:21,elem,-1].flatten())
    elif pron[i1] == 'min':
        val = min(favar.phisc[:21,elem,-1].flatten())        
    pos = list(favar.phisc[:21,elem,-1].flatten()).index(val)
    if tcode_dic[bnames[i1]] in [5,7]:
        anval = numpy.round(favar.trans_dic[bnames[i1]]['std']*val*100.0,3)
    else:
        anval = numpy.round(favar.trans_dic[bnames[i1]]['std']*val,3)        
    ax2.annotate(str(anval), xy=(pos, val), xytext=(-5,5), 
                textcoords='offset points', ha='center', va='bottom', color='k')
    '''
if glopt['title']: ax2.set_title(r'Commodity market effects from shock to political uncertainty',fontsize=glopt['title_fs'])
ax2.hlines(numpy.array([0,]*48),xmin=0,xmax=48,linestyles='solid',linewidth=hzline_width,colors='k')
plt.legend(loc="lower right")
fig2.savefig('../graphs/'+modname+'/'+'COMM.eps',bbox_inches='tight')


# Create flight to safety graph
fig2 = plt.figure()
bnames = ['TB3MS','TB6MS','GS1','GS2','GS3','GS5','GS7','GS10']
bcols = ['#000000','#302F30','#6B6B6B','#B5B3B4','#000000','#302F30','#6B6B6B','#B5B3B4']
pron = ['min','min','min','min','min','min','min','min']
btype = ['-','-','-','-','--','--','--','--']
indx = []
for namos in bnames:
    indx.append(favar.vnames.index(namos))
ax2 = fig2.add_subplot(1,1,1)
plt.grid()
for i1,elem in enumerate(indx):
    if tcode_dic[bnames[i1]] in [4,5,6,7,8]:
        percen = 100.0
    else:
        percen = 1.0
    ax2.plot(favar.phisc[:49,elem,-1].flatten()*percen,label=bnames[i1],color=bcols[i1],linestyle=btype[i1])
    ax2.set_ylabel('Percentage Deviation',fontsize=glopt['label_fs'])
    '''
    if pron[i1] == 'max':
        val = max(favar.phisc[:21,elem,-1].flatten())
    elif pron[i1] == 'min':
        val = min(favar.phisc[:21,elem,-1].flatten())        
    pos = list(favar.phisc[:21,elem,-1].flatten()).index(val)
    if tcode_dic[bnames[i1]] in [5,7]:
        anval = numpy.round(favar.trans_dic[bnames[i1]]['std']*val*100.0,3)
    else:
        anval = numpy.round(favar.trans_dic[bnames[i1]]['std']*val,3)        
    ax2.annotate(str(anval), xy=(pos, val), xytext=(-5,5), 
                textcoords='offset points', ha='center', va='bottom', color='k')
    '''
if glopt['title']: ax2.set_title(r'Bond market effects from shock to political uncertainty',fontsize=glopt['title_fs'])
ax2.hlines(numpy.array([0,]*48),xmin=0,xmax=48,linestyles='solid',linewidth=hzline_width,colors='k')
plt.legend(loc="lower right")
fig2.savefig('../graphs/'+modname+'/'+'SAFETY.eps',bbox_inches='tight')

# Create external finance premium graph
fig2 = plt.figure()
bnames = ['AAA','BAA','SPINDEX']
bcols = ['#000000','#302F30','#6B6B6B']
pron = ['max','min','min']
btype = ['--','--','-']
indx = []
for namos in bnames:
    indx.append(favar.vnames.index(namos))
ax2 = fig2.add_subplot(1,1,1)
ax21 = plt.twinx()
plt.grid()
linli = []
for i1,elem in enumerate(indx):
    if tcode_dic[bnames[i1]] in [4,5,6,7,8]:
        percen = 100.0
    else:
        percen = 1.0
    if bnames[i1] == 'SPINDEX':
        linli.append(ax2.plot(favar.phisc[:49,elem,-1].flatten()*percen,label=bnames[i1],color=bcols[i1],linestyle=btype[i1]))
    elif bnames[i1] == 'AAA' or bnames[i1] == 'BAA':
        linli.append(ax21.plot(favar.phisc[:49,elem,-1].flatten()*percen,label=bnames[i1],color=bcols[i1],linestyle=btype[i1]))
    ax2.set_ylabel('Percentage Deviation',fontsize=glopt['label_fs'])
    ax21.set_ylabel('Percentage Deviation',fontsize=glopt['label_fs'])
    '''
    if pron[i1] == 'max':
        val = max(favar.phisc[:21,elem,-1].flatten())
    elif pron[i1] == 'min':
        val = min(favar.phisc[:21,elem,-1].flatten())        
    pos = list(favar.phisc[:21,elem,-1].flatten()).index(val)
    if tcode_dic[bnames[i1]] in [5,7]:
        anval = numpy.round(favar.trans_dic[bnames[i1]]['std']*val*100.0,3)
    else:
        anval = numpy.round(favar.trans_dic[bnames[i1]]['std']*val,3)        
    ax2.annotate(str(anval), xy=(pos, val), xytext=(-5,5), 
                textcoords='offset points', ha='center', va='bottom', color='k')
    '''
if glopt['title']: ax2.set_title(r'External finance effect from shock to political uncertainty',fontsize=glopt['title_fs'])
ax2.hlines(numpy.array([0,]*48),xmin=0,xmax=48,linestyles='solid',linewidth=hzline_width,colors='k')
# added these three lines
labs = [l[0].get_label() for l in linli]
labs[0] = labs[0]+"      (right -->)"
labs[1] = labs[1]+"      (right -->)"
labs[2] = labs[2]+" (<-- left)"
ax2.legend([x[0] for x in linli], labs, loc="lower right")

fig2.savefig('../graphs/'+modname+'/'+'EXTFIN.eps',bbox_inches='tight')

# Create consumption-saving graph
fig2 = plt.figure()
bnames = ['PCES','PCEND','PCEDG','PCEPI','PSAVERT']
pron = ['min','min','min','min','max']
bcols = ['#000000','#302F30','#6B6B6B','#000000','#6B6B6B']
btype = ['-','-','-','--','--']
indx = []
for namos in bnames:
    indx.append(favar.vnames.index(namos))
ax2 = fig2.add_subplot(1,1,1)
plt.grid()
for i1,elem in enumerate(indx):
    if tcode_dic[bnames[i1]] in [4,5,6,7,8]:
        percen = 100.0
    else:
        percen = 1.0    
    ax2.plot(favar.phisc[:49,elem,-1].flatten()*percen,label=bnames[i1],color=bcols[i1],linestyle=btype[i1])
    ax2.set_ylabel('Percentage Deviation',fontsize=glopt['label_fs'])
    '''
    if pron[i1] == 'max':
        val = max(favar.phisc[:21,elem,-1].flatten())
    elif pron[i1] == 'min':
        val = min(favar.phisc[:21,elem,-1].flatten())        
    pos = list(favar.phisc[:21,elem,-1].flatten()).index(val)
    if tcode_dic[bnames[i1]] in [4,5,6,7,8]:
        anval = numpy.round(favar.trans_dic[bnames[i1]]['std']*val*100.0,3)
    else:
        anval = numpy.round(favar.trans_dic[bnames[i1]]['std']*val,3)        
    ax2.annotate(str(anval), xy=(pos, val), xytext=(-5,5), 
                textcoords='offset points', ha='center', va='bottom', color='k')
    '''
if glopt['title']: ax2.set_title(r'Consumption & saving effect from shock to political uncertainty',fontsize=glopt['title_fs'])
ax2.hlines(numpy.array([0,]*48),xmin=0,xmax=48,linestyles='solid',linewidth=hzline_width,colors='k')
plt.legend(loc="lower right")
fig2.savefig('../graphs/'+modname+'/'+'CONSSAV.eps',bbox_inches='tight')


fevdli = []
fevdli.append('UNCERT')
fevdli.append('IP')
fevdli.append('EMP')
fevdli.append('SPINDEX')
fevdli.append('PCES')
fevdli.append('PCEDG')
fevdli.append('PCEND')
fevdli.append('PCES')
fevdli.append('TB3MS')
fevdli.append('TB6MS')
fevdli.append('GS1')
fevdli.append('GS2')
fevdli.append('GS5')

fevdli = copy.deepcopy(vnames)

# Also create the table for the variance decomposition
tmpstr = ''
tmpstr = tmpstr + r"\begin{table}\centering"+"\n"
tmpstr = tmpstr + r"\caption{Contribution of Policy Uncertainty Shock to Variance of Forecasts\\Benchmark ordering}"+"\n"
tmpstr = tmpstr + r"\footnotesize"+"\n"
tmpstr = tmpstr + r"\begin{tabular*}{0.8\textwidth}{@{\extracolsep{\fill}}@{}llccccccc@{}}\hline\hline"+"\n"
tmpstr = tmpstr + r" & & \multicolumn{6}{c}{Variance Decomposition at h}"+r"\\"+"\n"
tmpstr = tmpstr + r"\cline{3-8}"+"\n"
tmpstr = tmpstr + r"No. & Variable & h=6 & h=12 & h=18 & h=24 & h=36 & h=60 & $R^2$"+r"\\"+"\n"
tmpstr = tmpstr + r"\hline"+"\n"
for i1,namo in enumerate(fevdli):
    tmpstr = tmpstr + str(i1) + r" & "
    tmpstr = tmpstr + namo
    val6 = numpy.round(favar.decomp[vnames.index(namo),5,-1]/100.0,3)
    val6s = str(val6)
    if len(val6s) == 4:
        val6s = val6s + '0'
    elif len(val6s) == 3:
        val6s = val6s + '00'    
    tmpstr = tmpstr + r"&"+ val6s
    val12 = numpy.round(favar.decomp[vnames.index(namo),11,-1]/100.0,3)
    val12s = str(val12)
    if len(val12s) == 4:
        val12s = val12s + '0'
    elif len(val12s) == 3:
        val12s = val12s + '00'    
    tmpstr = tmpstr + r"&"+ val12s    
    val18 = numpy.round(favar.decomp[vnames.index(namo),17,-1]/100.0,3)
    val18s = str(val18)
    if len(val18s) == 4:
        val18s = val18s + '0'
    elif len(val18s) == 3:
        val18s = val18s + '00'    
    tmpstr = tmpstr + r"&"+ val18s
    val24 = numpy.round(favar.decomp[vnames.index(namo),23,-1]/100.0,3)
    val24s = str(val24)
    if len(val24s) == 4:
        val24s = val24s + '0'
    elif len(val24s) == 3:
        val24s = val24s + '00'    
    tmpstr = tmpstr + r"&"+ val24s
    val36 = numpy.round(favar.decomp[vnames.index(namo),35,-1]/100.0,3)
    val36s = str(val36)
    if len(val36s) == 4:
        val36s = val36s + '0'
    elif len(val36s) == 3:
        val36s = val36s + '00'    
    tmpstr = tmpstr + r"&"+ val36s
    val60 = numpy.round(favar.decomp[vnames.index(namo),59,-1]/100.0,3)
    val60s = str(val60)
    if len(val60s) == 4:
        val60s = val60s + '0'
    elif len(val60s) == 3:
        val60s = val60s + '00'    
    tmpstr = tmpstr + r"&"+ val60s
    yy = favar.tdata_adj[:,vnames.index(namo)]
    xx = favar.fdata_adj
    betta = numpy.dot(numpy.linalg.inv(numpy.dot(xx.T,xx)),numpy.dot(xx.T,yy))
    yyfit = numpy.dot(xx,betta.T)
    RSS = numpy.sum((yy - yyfit)**2)
    TSS = numpy.sum((yy - numpy.mean(yy))**2)
    ESS = TSS - RSS
    R2 = numpy.round(ESS/TSS,3)
    sR2 = str(R2)
    if len(sR2) == 4:
        sR2 = sR2 + '0'
    elif len(sR2) == 3:
        sR2 = sR2 + '00'
    if namo != "UNCERT":
        tmpstr = tmpstr + r"&"+ sR2+r"\\"+"\n"
    else:
        tmpstr = tmpstr + r"&"+ "1.000"+r"\\"+"\n"
tmpstr = tmpstr + r"\hline\hline"+"\n"
tmpstr = tmpstr + r"\multicolumn{9}{l}{{\bf Notes:} Right-hand side column shows $R^2$ from regression of variables on factors.}\\"+"\n"
tmpstr = tmpstr + r"\end{tabular*}"+"\n"
tmpstr = tmpstr + r"\end{table}"+"\n"
if 'table5.tex' in os.listdir('../tex_papers/bloom_favar/'):
    os.remove('../tex_papers/bloom_favar/table5.tex')
f1 = open('../tex_papers/bloom_favar/table5.tex','w')
f1.write(tmpstr)
f1.close()



from ..utils import MATRIX, MATRIX_DICT
from .algebra import Matrix, MatrixBase, MatrixDict

from ..core import init_module
init_module.import_lowlevel_operations()

def MATRIX_DICT_iadd(self, other):
    """ Inplace matrix add.
    """
    t = type(other)
    if t is list or t is tuple:
        other = Matrix(other)
        t = type(other)
    if t is MatrixDict:
        if self.is_writable:
            ret = self
        else:
            ret = self.copy()
        head1, data1 = ret.pair
        head2, data2 = other.pair
        assert head1.shape==head2.shape,`head1, head2`
        if head1.is_transpose:
            if head2.is_transpose:
                iadd_MATRIX_MATRIX_TT(data1, data2)
            else:
                iadd_MATRIX_MATRIX_TA(data1, data2)
        elif head2.is_transpose:
            iadd_MATRIX_MATRIX_AT(data1, data2)
        else:
            iadd_MATRIX_MATRIX_AA(data1, data2)
        if head2.is_array:
            return ret.A
        return ret.M
    elif isinstance(other, MatrixBase):
        raise NotImplementedError(`type(other)`)
    else:
        if other:
            if self.is_writable:
                ret = self
            else:
                ret = self.copy()
            head, data = ret.pair
            rows, cols = head.shape
            if head.is_transpose:
                iadd_MATRIX_T_SCALAR(rows, cols, data, other)
            else:
                iadd_MATRIX_SCALAR(rows, cols, data, other)
            return ret
        else:
            return self

def MATRIX_DICT_imul(self, other):
    """ Inplace matrix multiplication.
    """
    t = type(other)
    if t is list or t is tuple:
        other = Matrix(other)
        t = type(other)
    if t is MatrixDict:
        head1, data1 = self.pair
        head2, data2 = other.pair
        if head1.is_array or head2.is_array:
            assert head1.shape==head2.shape,`head1, head2`
            if self.is_writable:
                ret = self
            else:
                ret = self.copy()
            data1 = ret.data
            if head1.is_transpose:
                if head2.is_transpose:
                    imul_MATRIX_MATRIX_ATT(data1, data2)
                else:
                    imul_MATRIX_MATRIX_TA(data1, data2)
            elif head2.is_transpose:
                imul_MATRIX_MATRIX_AT(data1, data2)
            else:
                imul_MATRIX_MATRIX_AA(data1, data2)
            if head2.is_array:
                return ret.A
            return ret.M
        else:
            assert head1.cols==head2.rows,`head1, head2`
            args = data1, data2, head1.rows, head2.cols, head1.cols
            if head1.is_transpose:
                if head2.is_transpose:
                    ret = mul_MATRIX_MATRIX_MTT(*args)
                else:
                    ret = mul_MATRIX_MATRIX_TM(*args)
            elif head2.is_transpose:
                ret = mul_MATRIX_MATRIX_MT(*args)
            else:
                ret = mul_MATRIX_MATRIX_MM(*args)
            return ret
    elif isinstance(other, MatrixBase):
        raise NotImplementedError(`type(other)`)
    else:
        if self.is_writable:
            ret = self
        else:
            ret = self.copy()
        if other:
            head, data = ret.pair
            for key in data:
                data[key] *= other
        else:
            head, data = ret.pair
            data.clear()
        return ret

def iadd_MATRIX_SCALAR(rows, cols, data, value):
    col_indices = range(cols)
    for i in xrange(rows):
        for j in col_indices:
            key = (i,j)
            b = data.get(key)
            if b is None:
                data[key] = value
            else:
                b += value
                if b:
                    data[key] = b
                else:
                    del data[key]

def iadd_MATRIX_T_SCALAR(rows, cols, data, value):
    col_indices = range(cols)
    for i in xrange(rows):
        for j in col_indices:
            key = (j,i)
            b = data.get(key)
            if b is None:
                data[key] = value
            else:
                b += value
                if b:
                    data[key] = b
                else:
                    del data[key]

def iadd_MATRIX_MATRIX_AA(data1, data2):
    for key,x in data2.items():
        b = data1.get(key)
        if b is None:
            data1[key] = x
        else:
            b += x
            if b:
                data1[key] = b
            else:
                del data1[key]

def iadd_MATRIX_MATRIX_AT(data1, data2):
    for (j,i),x in data2.items():
        key = i,j
        b = data1.get(key)
        if b is None:
            data1[key] = x
        else:
            b += x
            if b:
                data1[key] = b
            else:
                del data1[key]

def iadd_MATRIX_MATRIX_TA(data1, data2):
    for (i,j),x in data2.items():
        key = j,i
        b = data1.get(key)
        if b is None:
            data1[key] = x
        else:
            b += x
            if b:
                data1[key] = b
            else:
                del data1[key]

iadd_MATRIX_MATRIX_TT = iadd_MATRIX_MATRIX_AA

def imul_MATRIX_MATRIX_AA(data1, data2):
    for key in data1:
        b = data2.get(key)
        if b is None:
            del data1[key]
        else:
            data1[key] *= b

def imul_MATRIX_MATRIX_AT(data1, data2):
    for key in data1:
        i,j = key
        b = data2.get((j,i))
        if b is None:
            del data1[key]
        else:
            data1[key] *= b

imul_MATRIX_MATRIX_TA = imul_MATRIX_MATRIX_AT
imul_MATRIX_MATRIX_ATT = imul_MATRIX_MATRIX_AA

def mul_MATRIX_MATRIX_AA(data1, data2, rows, cols):
    indices = range(n)
    d = {}
    data1_get = data1.get
    data2_get = data2.get
    for i in xrange(rows):
        for j in xrange(cols):
            key = i,j
            a_ij = data1_get(key)
            if a_ij is None:
                continue
            b_ij = data2_get(key)
            if b_ij is None:
                continue
            d[key] = a_ij * b_ij
    return MatrixDict(MATRIX(rows, cols, MATRIX_DICT), d)

def mul_MATRIX_MATRIX_AT(data1, data2, rows, cols):
    indices = range(n)
    d = {}
    data1_get = data1.get
    data2_get = data2.get
    for i in xrange(rows):
        for j in xrange(cols):
            key = i,j
            a_ij = data1_get(key)
            if a_ij is None:
                continue
            b_ij = data2_get((j,i))
            if b_ij is None:
                continue
            d[key] = a_ij * b_ij
    return MatrixDict(MATRIX(rows, cols, MATRIX_DICT), d)

def mul_MATRIX_MATRIX_TA(data1, data2, rows, cols):
    indices = range(n)
    d = {}
    data1_get = data1.get
    data2_get = data2.get
    for i in xrange(rows):
        for j in xrange(cols):
            key = i,j
            a_ij = data1_get((j,i))
            if a_ij is None:
                continue
            b_ij = data2_get(key)
            if b_ij is None:
                continue
            d[key] = a_ij * b_ij
    return MatrixDict(MATRIX(rows, cols, MATRIX_DICT), d)

def mul_MATRIX_MATRIX_ATT(data1, data2, rows, cols):
    indices = range(n)
    d = {}
    data1_get = data1.get
    data2_get = data2.get
    for i in xrange(rows):
        for j in xrange(cols):
            ikey = j,i
            a_ij = data1_get(ikey)
            if a_ij is None:
                continue
            b_ij = data2_get(ikey)
            if b_ij is None:
                continue
            d[i,j] = a_ij * b_ij
    return MatrixDict(MATRIX(rows, cols, MATRIX_DICT), d)

def mul_MATRIX_MATRIX_MM(data1, data2, rows, cols, n):
    d = {}
    left_indices = {}
    right_indices = {}
    for i,j in data1.keys():
        s = left_indices.get(i)
        if s is None:
            s = left_indices[i] = set()
        s.add(j)
    for j,k in data2.keys():
        s = right_indices.get(k)
        if s is None:
            s = right_indices[k] = set()
        s.add(j)

    for i,ji in left_indices.items ():
        for k,jk in right_indices.items ():
            for j in ji.intersection(jk):
                dict_add_item(None, d, (i,k), data1[(i,j)] * data2[(j,k)])
    return MatrixDict(MATRIX(rows, cols, MATRIX_DICT), d)

def mul_MATRIX_MATRIX_TM(data1, data2, rows, cols, n):
    d = {}
    left_indices = {}
    right_indices = {}
    for j,i in data1.keys():
        s = left_indices.get(i)
        if s is None:
            s = left_indices[i] = set()
        s.add(j)
    for j,k in data2.keys():
        s = right_indices.get(k)
        if s is None:
            s = right_indices[k] = set()
        s.add(j)

    for i,ji in left_indices.items ():
        for k,jk in right_indices.items ():
            for j in ji.intersection(jk):
                dict_add_item(None, d, (i,k), data1[(j,i)] * data2[(j,k)])
    return MatrixDict(MATRIX(rows, cols, MATRIX_DICT), d)

def mul_MATRIX_MATRIX_MT(data1, data2, rows, cols, n):
    d = {}
    left_indices = {}
    right_indices = {}
    for i,j in data1.keys():
        s = left_indices.get(i)
        if s is None:
            s = left_indices[i] = set()
        s.add(j)
    for k,j in data2.keys():
        s = right_indices.get(k)
        if s is None:
            s = right_indices[k] = set()
        s.add(j)

    for i,ji in left_indices.items ():
        for k,jk in right_indices.items ():
            for j in ji.intersection(jk):
                dict_add_item(None, d, (i,k), data1[(i,j)] * data2[(k,j)])
    return MatrixDict(MATRIX(rows, cols, MATRIX_DICT), d)

def mul_MATRIX_MATRIX_MTT(data1, data2, rows, cols, n):
    d = {}
    left_indices = {}
    right_indices = {}
    for j,i in data1.keys():
        s = left_indices.get(i)
        if s is None:
            s = left_indices[i] = set()
        s.add(j)
    for k,j in data2.keys():
        s = right_indices.get(k)
        if s is None:
            s = right_indices[k] = set()
        s.add(j)

    for i,ji in left_indices.items ():
        for k,jk in right_indices.items ():
            for j in ji.intersection(jk):
                dict_add_item(None, d, (i,k), data1[(j,i)] * data2[(k,j)])
    return MatrixDict(MATRIX(rows, cols, MATRIX_DICT), d)





__all__ = ['SUBSCRIPT', 'SLICE', 'LAMBDA', 'ATTR', 'KWARG']

from .base import Head, heads_precedence

from ..core import init_module
init_module.import_heads()

class FunctionalHead(Head):

    # Derived classes must define `parenthesis` data member
    
    def data_to_str_and_precedence(self, cls, (func, args)):
        o_p = getattr(heads_precedence, repr(self))
        arg_p = heads_precedence.ARG        
        f, f_p = func.head.data_to_str_and_precedence(type(func), func.data)
        if f_p < o_p: f = '('+f+')'
        l = []
        assert type(args) in [tuple], `args`
        for arg in args:
            a, a_p = arg.head.data_to_str_and_precedence(cls, arg.data)
            if a_p < arg_p: a = '('+a+')'
            l.append(a)
        p1,p2 = self.parenthesis
        f = f + p1 + ', '.join(l) + p2
        return f,o_p


class SubscriptHead(FunctionalHead):
    """
    SubscriptHead is a head for n-ary subscript operation,
    data is a (1+n)-tuple of expression operands
    """

    op_mth = '__getitem__'
    parenthesis = '[]'

    def is_data_ok(self, cls, data):
        if type(data) is tuple:
            if len(data)<2:
                return '%s data part must be a 2-tuple but got %s-tuple' % (self, len(data))
            base, items = data
            if not isinstance(base, cls):
                return '%s data 1st item must be %s instance but got %s' % (self, cls, type(a))
            if type(items) is not tuple:
                return '%s data 2nd item must be a tuple but got %s' % (self, type(items))
            for i,a in enumerate(items):
                if not isinstance(a, cls):
                    return '%s data 2nd items %sth item must be %s instance but got %s' % (self, i, cls, type(a))
        else:
            return '%s data part must be a tuple but got %s' % (self, type(data))

    def __repr__(self): return 'SUBSCRIPT'

    def walk(self, func, cls, (base, items), target):
        new_base = base.head.walk(func, cls, base.data, base)
        flag = new_base is not base
        new_items = []
        for op in items:
            o = op.head.walk(func, cls, op.data, op)
            if op is not o:
                flag = True
            new_items.append(o)
        if flag:
            r = self.new(cls, (new_base, tuple(new_items)))
            return func(cls, r.head, r.data, r)
        return func(cls, self, (base, items), target)



class SliceHead(Head):
    """
    SliceHead is a head for 3-ary slice operation,
    data is a 3-tuple of expression operands
    """

    def data_to_str_and_precedence(self, cls, (start, stop, step)):
        slice_p = heads_precedence.SLICE
        colon_p = heads_precedence.COLON
        h, d = start.pair
        if h is SPECIAL and d is None:
            start = None
        else:
            start, p = h.data_to_str_and_precedence(cls, d)
            if p < colon_p: start = '('+start+')'
        h, d = stop.pair
        if h is SPECIAL and d is None:
            stop = None
        else:
            stop, p = h.data_to_str_and_precedence(cls, d)
            if p < colon_p: stop = '('+stop+')'
        h, d = step.pair
        if h is SPECIAL and d is None:
            step = None
        else:
            step, p = h.data_to_str_and_precedence(cls, d)
            if p < colon_p: step = '('+step+')'
        if start is None:
            if stop is None:
                if step is None: r = ':'
                else: r = '::%s' % step
            else:
                if step is None: r = ':%s' % stop
                else: r = ':%s:%s' % (stop, step)
        else:
            if stop is None:
                if step is None: r = '%s:' % start
                else: r = '%s::%s' % (start, step)
            else:
                if step is None: r ='%s:%s' % (start, stop)
                else: r = '%s:%s:%s' % (start, stop, step)
        return r, slice_p


    def __repr__(self): return 'SLICE'

class LambdaHead(Head):
    """
    LambdaHead is a head for lambda expression,
    data is a 2-tuple of arguments and an expression.
    """
    def data_to_str_and_precedence(self, cls, (args, expr)):
        arg_p = heads_precedence.ARG
        colon_p = heads_precedence.COLON
        l = []
        assert type(args) in [tuple],`args`
        for arg in args:
            a, a_p = arg.head.data_to_str_and_precedence(cls, arg.data)
            if a_p < arg_p: a = '('+a+')'
            l.append(a)
        e, e_p = expr.head.data_to_str_and_precedence(cls, expr.data)
        if e_p < colon_p: e = '('+e+')'
        return 'lambda %s: %s' % (', '.join(l), e), heads_precedence.LAMBDA

    def __repr__(self): return 'LAMBDA'

class AttrHead(Head):
    """
    AttrHead is a head for attribute operation,
    data is a 2-tuple of expression operands.
    """

    def data_to_str_and_precedence(self, cls, (expr, attr)):
        dot_p = heads_precedence.DOT
        e, e_p = expr.head.data_to_str_and_precedence(cls, expr.data)
        if e_p < dot_p: e = '('+e+')'
        a, a_p = attr.head.data_to_str_and_precedence(cls, attr.data)
        if a_p < dot_p: a = '('+a+')'
        return e + '.' + a, heads_precedence.ATTR
            
    def __repr__(self): return 'ATTR'

class KwargHead(Head):
    """
    Kwarg is a head for keyword argument,
    data is a 2-tuple of expression operands.
    """
    def data_to_str_and_precedence(self, cls, (name, expr)):
        kw_p = heads_precedence.KWARG
        assign_p = heads_precedence.ASSIGN
        n, n_p = name.head.data_to_str_and_precedence(cls, name.data)
        if n_p < assign_p: n = '('+n+')'
        e, e_p = expr.head.data_to_str_and_precedence(cls, expr.data)
        if e_p < assign_p: e = '('+e+')'
        return n + '=' + e, kw_p

    def __repr__(self): return 'KWARG'


SUBSCRIPT = SubscriptHead()
SLICE = SliceHead()
LAMBDA = LambdaHead()
ATTR = AttrHead()
KWARG = KwargHead()



'''
.. module:: solvers
   :platform: Linux
   :synopsis: Wrapper package for all kinds of modules necessary for finding solutions to DSGE model instances. Contains modules
              required for finding steady state solutions as well as such which are needed for solving the models dynamically.

.. moduleauthor:: Eric M. Scheffel <eric.scheffel@nottingham.edu.cn>


'''


#
# Created January 2008 by Fredrik Johansson
#
""" Provides elementary calculus functions sqrt, exp, log, sin, etc and constants pi, E.
"""

__all__ = ['Sqrt', 'Exp', 'Log', 'Sin', 'Cos', 'Tan', 'Cot', 'Sign', 'Mod', 'Factorial',
           'E', 'pi', 'gamma', 'Ln', 'ArcSin']
__docformat__ = "restructuredtext"

from ...core import init_module
init_module.import_heads()
init_module.import_numbers()

from ..algebra import Calculus, I
from ..infinity import oo, zoo, undefined, CalculusInfinity
from ..constants import const_pi, const_E, const_gamma
from ...utils import str_SUM, str_SYMBOL

from ...core import DefinedFunction, get_nargs
from ...arithmetic.evalf import evalf
from ...arithmetic.numbers import complextypes, realtypes, inttypes
from ...arithmetic.number_theory import factorial
from ...arithmetic import infinity

from .algebra import CalculusFunctionRing

Apply = Calculus.Apply

import math

E = const_E.as_algebra(Calculus)
pi = const_pi.as_algebra(Calculus)
gamma = const_gamma.as_algebra(Calculus)

@init_module
def _init(m):
    m.zero = Calculus.zero
    m.one = Calculus.one
    m.half = one/2
    sqrt2 = Calculus.convert('2**(1/2)')
    sqrt3 = Calculus.convert('3**(1/2)')

    m.Ipi = Ipi = I*pi
    m.Ipi2 = Ipi2 = Ipi/2

    m.log_number_table = {
        m.zero.data : -oo,
        m.one.data : m.zero,
        I.data : Ipi2,
        (-I).data : -Ipi2
        }

    # Tabulated values of sin(x) at multiples of pi/12
    C0 = (sqrt3-1)/(2*sqrt2)
    C1 = one/2
    C2 = sqrt2/2
    C3 = sqrt3/2
    C4 = (sqrt3+1)/(2*sqrt2)
    
    # Replace entries with None to prevent from evaluating
    m.sine_table = [ \
  Calculus.zero, C0, C1, C2, C3, C4, Calculus.one, C4, C3, C2, C1,C0,
  Calculus.zero,-C0,-C1,-C2,-C3,-C4,-Calculus.one,-C4,-C3,-C2,-C1,-C0]

class CalculusDefinedFunction(DefinedFunction):

    @classmethod
    def get_argument_algebras(cls):
        return (Calculus,)*(get_nargs(cls))

class Sign(CalculusDefinedFunction):
    def __new__(cls, arg):
        if not isinstance(arg, Calculus):
            arg = Calculus.convert(arg)
        return Apply(cls, arg)

class Mod(CalculusDefinedFunction):
    def __new__(cls, x, y):
        if not isinstance(x, Calculus):
            x = Calculus.convert(x)
        if not isinstance(y, Calculus):
            y = Calculus.convert(y)
        xh,xd = x.pair
        yh,yd = y.pair
        if xh is NUMBER and yh is NUMBER:
            return Calculus.convert(xd % yd)
        return Apply(cls, (x, y))

class Factorial(CalculusDefinedFunction):
    
    def __new__(cls, x):
        if not isinstance(x, Calculus):
            x = Calculus.convert(x)
        if x.head is NUMBER:
            n = x.data
            if isinstance(n, (int, long)):
                if n >= 0:
                    return Calculus.convert(factorial(n))
                return zoo
        return Apply(cls, x)

#---------------------------------------------------------------------------#
#                                  Exponentials                             #
#---------------------------------------------------------------------------#

class Sqrt(CalculusDefinedFunction):
    def __new__(cls, arg):
        return arg ** half

class Exp(CalculusDefinedFunction):
    def __new__(cls, arg):
        return E ** arg

    @staticmethod
    def derivative(arg):
        return E ** arg

    @classmethod
    def fdiff(cls, FAlgebra, argument_index, order):
        if argument_index!=0:
            if isinstance(argument_index, inttypes):
                raise TypeError('%s.fdiff argument_index must be 0 but got %r' % (cls.__name__, argument_index))
            return NotImplemented
        return FAlgebra(CALLABLE, cls)

class Log(CalculusDefinedFunction):
    def __new__(cls, arg, base=E):
        if type(arg) is not Calculus:
            if isinstance(arg, CalculusInfinity):
                if arg == oo:
                    return oo
                if arg == undefined:
                    return undefined
                return Apply(cls, arg)
            else:
                arg = Calculus.convert(arg)
        if base != E:
            base = Calculus.convert(base)
            bd = base.data
            ad = arg.data
            if base.head is NUMBER and isinstance(bd, inttypes) and \
                arg.head is NUMBER and isinstance(ad, inttypes) and \
                ad > 0 and bd > 1:
                l = int(math.log(ad, bd) + 0.5)
                if bd**l == ad:
                    return Calculus.convert(l)
            return cls(arg) / cls(base)
        head = arg.head
        data = arg.data
        if head is NUMBER:
            v = log_number_table.get(data)
            if v is not None:
                return v
            if isinstance(data, realtypes) and data < 0:
                return Ipi + Log(-arg)
            if isinstance(data, complextypes) and data.real == 0:
                im = data.imag
                if im > 0: return Apply(cls, Calculus(NUMBER, im)) + Ipi2
                if im < 0: return Apply(cls, Calculus(NUMBER, -im)) - Ipi2
            return Apply(cls, arg)
        if arg == E:
            return one
        from ..relational import is_positive
        if head is POW:
            base, expt = data
            if is_positive(base) and isinstance(expt, realtypes):
                return Apply(cls, base) * expt
        if head is TERM_COEFF:
            term, coeff = data
            if (isinstance(coeff, realtypes) and coeff < 0) and is_positive(base):
                return Ipi + Log(-arg)
        return Apply(cls, arg)

    @classmethod
    def derivative(cls, arg):
        return 1/arg

    @classmethod
    def nth_derivative(cls, arg, n=1):
        return (-1)**(n-1) * Factorial(n-1) * arg**(-n)

    @classmethod
    def fdiff(cls, FAlgebra, argument_index, order):
        if argument_index!=0:
            if isinstance(argument_index, inttypes):
                raise TypeError('%s.fdiff argument_index must be 0 but got %r' % (cls.__name__, argument_index))
            return NotImplemented
        if order==1:
            func = lambda x: 1/x
        else:
            func = lambda x: (-1)**(order-1) * Factorial(order-1) * x**(-order)
        return FAlgebra(CALLABLE, func)

class Ln(Log):
    def __new__(cls, arg):
        return Log(arg)

#---------------------------------------------------------------------------#
#                          Trigonometric functions                          #
#---------------------------------------------------------------------------#



def get_pi_shift(arg, N):
    """Parse as x, n where arg = x + n*pi/N"""
    if arg.head is TERM_COEFF:
        e, c = arg.data
        if e == pi:
            c *= N
            if isinstance(c, inttypes):
                return zero, c
    elif arg.head is TERM_COEFF_DICT:
        c = arg.data.get(pi)
        if c is not None:
            c *= N
            if isinstance(c, inttypes):
                d = arg.data.copy()
                d.pop(pi)
                return TERM_COEFF_DICT.new(type(arg), d), c
    elif arg.head is ADD:
        rest = []
        x1 = None
        for op in arg.data:
            x, n = get_pi_shift(op, N)
            if n:
                x1, n1 = x, n
                assert x1==0,`x1,n1`
            else:
                rest.append(op)
        if x1 is not None:
            return ADD.new(type(arg), rest), n1
    elif arg == pi:
        return zero, N
    return arg, 0

def is_odd (expr):
    if isinstance(expr, int) and expr % 2:
        return True
    return False

def has_leading_sign(arg):
    """Detect symmetry cases for odd/even functions."""
    if arg.head is NUMBER:
        if arg < 0:
            return True
    if arg.head is TERM_COEFF:
        if arg.data[1] < 0:
            return True
        return has_leading_sign(arg.data[0])
    if arg.head is TERM_COEFF_DICT:
        for t,c in arg.data.iteritems():
            if not c < 0:
                return False
        return True
    if arg.head is BASE_EXP_DICT:
        for b,e in arg.data.iteritems():
            if is_odd(e):
                if has_leading_sign(b):
                    return True
    return None

class TrigonometricFunction(CalculusDefinedFunction):

    parity = None   # 'even' or 'odd'
    period = None   # multiple of pi

    def __new__(cls, arg):
        if type(arg) is not Calculus:
            if isinstance(arg, CalculusInfinity):
                if arg == undefined:
                    return undefined
                arg = Calculus(SPECIAL, arg)
                return Apply(cls, arg)
            else:
                arg = Calculus.convert(arg)
        head, data = arg.pair
        if head is APPLY:
            func, args = data
            if func == cls.get_inverse_function():
                assert len (args)==1,`args`
                return args[0]

        x, m = get_pi_shift(arg, 12)
        m %= (12*cls.period)
        if x == zero:
            v = cls.eval_direct(m)
            if v is not None:
                return v
        period = cls.period
        negate_result = False
        # Full-period symmetry
        if not m % (12*period):
            arg = x
        else:
            # Half-period symmetry
            if not m % 12:
                arg = x
                negate_result = True
            # Quarter-period symmetry
            elif not m % 6:
                f = conjugates[cls]
                sign = (-1)**((((m-6)//12) % period) + (f.parity == 'odd'))
                return sign * f(x)
        if has_leading_sign(arg):
            arg = -arg
            negate_result ^= (cls.parity == 'odd')
        if negate_result:
            return -Apply(cls, arg)
        else:
            return Apply(cls, arg)

    @classmethod
    def get_inverse_function(cls):
        return

class Sin(TrigonometricFunction):
    parity = 'odd'
    period = 2

    @classmethod
    def eval_direct(cls, m):
        return sine_table[m % 24]

    @classmethod
    def derivative(cls, arg, n=1):
        if n == 1:
            return Cos(arg)

    @classmethod
    def nth_derivative(cls, arg, n):
        return Sin(arg + n*pi/2)

    @classmethod
    def fdiff(cls, FAlgebra, argument_index, order):
        if argument_index!=0:
            if isinstance(argument_index, inttypes):
                raise TypeError('%s.fdiff argument_index must be 0 but got %r' % (cls.__name__, argument_index))
            return NotImplemented
        if isinstance(order, inttypes):
            n = order % 4
            if n==0:
                return FAlgebra(CALLABLE, Sin)
            if n==1:
                return FAlgebra(CALLABLE, Cos)
            if n==2:
                return -FAlgebra(CALLABLE, Sin)
            return -FAlgebra(CALLABLE, Cos)
        order = order.data
        func = lambda x: Sin(x + order * pi/2)
        return FAlgebra(CALLABLE, func)

    @classmethod
    def get_inverse_function(cls):
        return CalculusFunctionRing(CALLABLE, ArcSin)

class Cos(TrigonometricFunction):
    parity = 'even'
    period = 2

    @classmethod
    def eval_direct(cls, m):
        return sine_table[(m+6) % 24]

    @classmethod
    def derivative(cls, arg, n=1):
        return -Sin(arg)

    @classmethod
    def nth_derivative(cls, arg, n=1):
        return Cos(arg + n*pi/2)

    @classmethod
    def fdiff(cls, FAlgebra, argument_index, order):
        if argument_index!=0:
            if isinstance(argument_index, inttypes):
                raise TypeError('%s.fdiff argument_index must be 0 but got %r' % (cls.__name__, argument_index))
            return NotImplemented
        if isinstance(order, inttypes):
            n = (order+1) % 4
            if n==0:
                return FAlgebra(CALLABLE, Sin)
            if n==1:
                return FAlgebra(CALLABLE, Cos)
            if n==2:
                return -FAlgebra(CALLABLE, Sin)
            return -FAlgebra(CALLABLE, Cos)
        func = lambda x: Cos(x + order * pi/2)
        return FAlgebra(CALLABLE, func)

class Tan(TrigonometricFunction):
    parity = 'odd'
    period = 1

    @classmethod
    def eval_direct(cls, m):
        a = sine_table[m % 24]
        b = sine_table[(m+6) % 24]
        if a is None or b is None:
            return
        return a / b

    @classmethod
    def derivative(cls, arg):
        return 1+Tan(arg)**2

    @classmethod
    def fdiff(cls, FAlgebra, argument_index, order):
        if argument_index!=0:
            if isinstance(argument_index, inttypes):
                raise TypeError('%s.fdiff argument_index must be 0 but got %r' % (cls.__name__, argument_index))
            return NotImplemented
        if order==1:
            tan = FAlgebra(CALLABLE, cls)
            return tan**2 + 1
        return NotImplemented

class Cot(TrigonometricFunction):
    parity = 'odd'
    period = 1

    @classmethod
    def eval_direct(cls, m):
        a = sine_table[m % 24]
        b = sine_table[(m+6) % 24]
        if a is None or b is None:
            return
        return b / a

    @classmethod
    def derivative(cls, arg, n=1):
        return -(1+Cot(arg)**2)

    @classmethod
    def fdiff(cls, FAlgebra, argument_index, order):
        if argument_index!=0:
            if isinstance(argument_index, inttypes):
                raise TypeError('%s.fdiff argument_index must be 0 but got %r' % (cls.__name__, argument_index))
            return NotImplemented
        if order==1:
            cot = FAlgebra(CALLABLE, cls)
            return -cot**2 - 1
        return NotImplemented

# pi/2-x symmetry
conjugates = {
  Sin : Cos,
  Cos : Sin,
  Tan : Cot,
  Cot : Tan
}

class ArcSin(CalculusDefinedFunction):
    def __new__(cls, x):
        if not isinstance(x, Calculus):
            x = Calculus.convert(x)
        return Apply(cls, x)



from pymaclab.filters._hpfilter import hpfilter as hpf
import numpy as np

def hpfilter(data=None,lam=1600):
    if type(data) == type(np.matlib.matrix([1,2,3])) and len(data.shape) == 2:
        if data.shape[0] < data.shape[1]: data = data.__array__().T
        else: data = data.__array__()
    elif type(data) != type(np.array([1,2,3])):
        data = np.array([x for x in data])
    elif type(data) == type(np.array([1,2,3])):
        if len(data.shape) == 2 and data.shape[0] < data.shape[1]:
            data = data.reshape(data.shape[1])
        elif len(data.shape) == 2 and data.shape[0] > data.shape[1]:
            data = data.reshape(data.shape[0])
    tlen = len(data)
    wdata = np.zeros((tlen,3))
    lam = 1600    
    return hpf(data,wdata,tlen,lam,0)



from sympycore import *

def test_combine_add_list():
    combine_add_list = heads.ADD.combine_add_list
    A = Ring
    x,y,z = map(A, 'xyz')
    assert combine_add_list(A, [x]) == [x]
    assert combine_add_list(A, [x,-x]) == []
    assert combine_add_list(A, [x,y]) == [x,y]
    assert combine_add_list(A, [x,x]) == [2*x]
    assert combine_add_list(A, [x,y,x]) == [x,y,x]
    assert combine_add_list(A, [x,y,-y]) == [x]
    assert combine_add_list(A, [x,y,-y,x]) == [2*x]
    assert combine_add_list(A, [-x,y,-y,x]) == []
    assert combine_add_list(A, [x,3*x]) == [4*x]
    assert combine_add_list(A, [2*x,3*x]) == [5*x]
    assert combine_add_list(A, [2*x,3*x,x]) == [6*x]



"""
Generated arithmetic methods for numbers.

This file is generated by the src/mk_numbers_methods.py script.
See http://sympycore.googlecode.com/ for more information.

DO NOT CHANGE THIS FILE DIRECTLY!!!
"""

from .numbers import mpqc, mpf, mpq, mpc, inttypes_set, realtypes_set, complextypes_set, numbertypes_set



def fraction_add(self, other, cls=mpq, inttypes_set=inttypes_set):
    t = type(other)
    #IF_CHECK_INT(T=t)
    if t in inttypes_set:
        #ADD_FRACTION_INT(LHS=self; RHS=other)
        p, q = self
        #RETURN_FRACTION(NUMER=p+q*(other); DENOM=q)
        return cls((p+q*(other), q))
    elif t is cls:
        #ADD_FRACTION_FRACTION(LHS=self; RHS=other)
        #ADDOP_FRACTION_FRACTION(LHS=self; RHS=other; SIGN=+; MOD=%)
        p, q = self
        r, s = other
        _tmp30 = p*s + q*r
        if not _tmp30:
            return 0
        #RETURN_FRACTION2(NUMER=_tmp30; DENOM=q*s; MOD=%)
        #FRACTION_NORMALIZE(NUMER=_tmp30; DENOM=q*s; RNUMER=_p; RDENOM=_q; MOD=%)
        _p = _x = _tmp30
        _q = _y = q*s
        while 1:
            _x, _y = _y, _x % _y
            if not _y:
                break
        if _x != 1:
            _p //= _x
            _q //= _x
        if _q == 1:
            return _p
        #RETURN_FRACTION(NUMER=_p; DENOM=_q)
        return cls((_p, _q))
    #IF_CHECK_FLOAT(T=t)
    if t is float:
        return self + mpf(other)
    return NotImplemented

def fraction_sub(self, other, cls=mpq, inttypes_set=inttypes_set):
    t = type(other)
    #IF_CHECK_INT(T=t)
    if t in inttypes_set:
        #SUB_FRACTION_INT(LHS=self; RHS=other)
        p, q = self
        #RETURN_FRACTION(NUMER=p-q*(other); DENOM=q)
        return cls((p-q*(other), q))
    elif t is cls:
        #SUB_FRACTION_FRACTION(LHS=self; RHS=other)
        #ADDOP_FRACTION_FRACTION(LHS=self; RHS=other; SIGN=-; MOD=%)
        p, q = self
        r, s = other
        _tmp93 = p*s - q*r
        if not _tmp93:
            return 0
        #RETURN_FRACTION2(NUMER=_tmp93; DENOM=q*s; MOD=%)
        #FRACTION_NORMALIZE(NUMER=_tmp93; DENOM=q*s; RNUMER=_p; RDENOM=_q; MOD=%)
        _p = _x = _tmp93
        _q = _y = q*s
        while 1:
            _x, _y = _y, _x % _y
            if not _y:
                break
        if _x != 1:
            _p //= _x
            _q //= _x
        if _q == 1:
            return _p
        #RETURN_FRACTION(NUMER=_p; DENOM=_q)
        return cls((_p, _q))
    #IF_CHECK_FLOAT(T=t)
    if t is float:
        return self - mpf(other)
    return NotImplemented

def fraction_rsub(self, other, cls=mpq, inttypes_set=inttypes_set):
    t = type(other)
    #IF_CHECK_INT(T=t)
    if t in inttypes_set:
        #SUB_INT_FRACTION(RHS=self; LHS=other)
        p, q = self
        #RETURN_FRACTION(NUMER=q*(other) - p; DENOM=q)
        return cls((q*(other) - p, q))
    #IF_CHECK_FLOAT(T=t)
    if t is float:
        return mpf(other) - self
    return NotImplemented

def fraction_mul(self, other, cls=mpq, inttypes_set=inttypes_set):
    t = type(other)
    if t is cls:
        #MUL_FRACTION_FRACTION(LHS=self; RHS=other; MOD=%)
        p, q = self
        r, s = other
        #RETURN_FRACTION2(NUMER=p*r; DENOM=q*s; MOD=%)
        #FRACTION_NORMALIZE(NUMER=p*r; DENOM=q*s; RNUMER=_p; RDENOM=_q; MOD=%)
        _p = _x = p*r
        _q = _y = q*s
        while 1:
            _x, _y = _y, _x % _y
            if not _y:
                break
        if _x != 1:
            _p //= _x
            _q //= _x
        if _q == 1:
            return _p
        #RETURN_FRACTION(NUMER=_p; DENOM=_q)
        return cls((_p, _q))
    #IF_CHECK_INT(T=t)
    if t in inttypes_set:
        #MUL_FRACTION_INT(LHS=self; RHS=other; MOD=%)
        p, q = self
        #RETURN_FRACTION2(NUMER=p*other; DENOM=q; MOD=%)
        #FRACTION_NORMALIZE(NUMER=p*other; DENOM=q; RNUMER=_p; RDENOM=_q; MOD=%)
        _p = _x = p*other
        _q = _y = q
        while 1:
            _x, _y = _y, _x % _y
            if not _y:
                break
        if _x != 1:
            _p //= _x
            _q //= _x
        if _q == 1:
            return _p
        #RETURN_FRACTION(NUMER=_p; DENOM=_q)
        return cls((_p, _q))
    #IF_CHECK_FLOAT(T=t)
    if t is float:
        return self * mpf(other)
    return NotImplemented

def fraction_div(self, other, cls=mpq, inttypes_set=inttypes_set):
    t = type(other)
    #IF_CHECK_INT(T=t)
    if t in inttypes_set:
        #DIV_FRACTION_INT(LHS=self; RHS=other; MOD=%)
        p, q = self
        #RETURN_FRACTION2(NUMER=p; DENOM=q*other; MOD=%)
        #FRACTION_NORMALIZE(NUMER=p; DENOM=q*other; RNUMER=_p; RDENOM=_q; MOD=%)
        _p = _x = p
        _q = _y = q*other
        while 1:
            _x, _y = _y, _x % _y
            if not _y:
                break
        if _x != 1:
            _p //= _x
            _q //= _x
        if _q == 1:
            return _p
        #RETURN_FRACTION(NUMER=_p; DENOM=_q)
        return cls((_p, _q))
    elif t is cls:
        #DIV_FRACTION_FRACTION(LHS=self; RHS=other; MOD=%)
        p, q = self
        r, s = other
        #RETURN_FRACTION2(NUMER=p*s; DENOM=q*r; MOD=%)
        #FRACTION_NORMALIZE(NUMER=p*s; DENOM=q*r; RNUMER=_p; RDENOM=_q; MOD=%)
        _p = _x = p*s
        _q = _y = q*r
        while 1:
            _x, _y = _y, _x % _y
            if not _y:
                break
        if _x != 1:
            _p //= _x
            _q //= _x
        if _q == 1:
            return _p
        #RETURN_FRACTION(NUMER=_p; DENOM=_q)
        return cls((_p, _q))
    #IF_CHECK_FLOAT(T=t)
    if t is float:
        return self / mpf(other)
    return NotImplemented

def fraction_rdiv(self, other, cls=mpq, inttypes_set=inttypes_set):
    t = type(other)
    #IF_CHECK_INT(T=t)
    if t in inttypes_set:
        #DIV_INT_FRACTION(RHS=self; LHS=other; MOD=%)
        p, q = self
        #RETURN_FRACTION2(NUMER=other*q; DENOM=p; MOD=%)
        #FRACTION_NORMALIZE(NUMER=other*q; DENOM=p; RNUMER=_p; RDENOM=_q; MOD=%)
        _p = _x = other*q
        _q = _y = p
        while 1:
            _x, _y = _y, _x % _y
            if not _y:
                break
        if _x != 1:
            _p //= _x
            _q //= _x
        if _q == 1:
            return _p
        #RETURN_FRACTION(NUMER=_p; DENOM=_q)
        return cls((_p, _q))
    #IF_CHECK_FLOAT(T=t)
    if t is float:
        return mpf(other) / self
    return NotImplemented

def fraction_pow(self, other, m=None, cls=mpq, inttypes_set=inttypes_set):
    t = type(other)
    #IF_CHECK_INT(T=t)
    if t in inttypes_set:
        if not other:
            return 1
        if other==1:
            return self
        #POW_FRACTION_INT(LHS=self; RHS=other)
        _tmp345 = other
        p, q = self
        if _tmp345 > 0:
            #RETURN_FRACTION(NUMER=p**_tmp345; DENOM=q**_tmp345)
            return cls((p**_tmp345, q**_tmp345))
        _tmp345 = -_tmp345
        if p > 0:
            #RETURN_FRACTION(NUMER=q**_tmp345; DENOM=p**_tmp345)
            return cls((q**_tmp345, p**_tmp345))
        #RETURN_FRACTION(NUMER=(-q)**_tmp345; DENOM=(-p)**_tmp345)
        return cls(((-q)**_tmp345, (-p)**_tmp345))
    #IF_CHECK_FLOAT(T=t)
    if t is float:
        return self ** mpf(other)
    return NotImplemented

def complex_add(self, other, new=object.__new__, cls=mpqc, realtypes_set=realtypes_set, complextypes_set=complextypes_set):
    t = type(other)
    #IF_CHECK_REAL(T=t)
    if t in realtypes_set:
        #ADD_COMPLEX_REAL(LHS=self; RHS=other)
        #RETURN_COMPLEX(REAL=self.real + other; IMAG=self.imag)
        _tmp394 = new(cls)
        _tmp394.real = self.real + other
        _tmp394.imag = self.imag
        return _tmp394
    #IF_CHECK_COMPLEX(T=t)
    if t in complextypes_set:
        #ADD_COMPLEX_COMPLEX(LHS=self; RHS=other)
        #RETURN_COMPLEX2(REAL=self.real + other.real; IMAG=self.imag + other.imag)
        _tmp415 = self.imag + other.imag
        if not _tmp415:
            return self.real + other.real
        #RETURN_COMPLEX(REAL=self.real + other.real; IMAG=_tmp415)
        _tmp422 = new(cls)
        _tmp422.real = self.real + other.real
        _tmp422.imag = _tmp415
        return _tmp422
    return NotImplemented

def complex_sub(self, other, new=object.__new__, cls=mpqc, realtypes_set=realtypes_set, complextypes_set=complextypes_set):
    t = type(other)
    #IF_CHECK_REAL(T=t)
    if t in realtypes_set:
        #SUB_COMPLEX_REAL(LHS=self; RHS=other)
        #RETURN_COMPLEX(REAL=self.real - other; IMAG=self.imag)
        _tmp443 = new(cls)
        _tmp443.real = self.real - other
        _tmp443.imag = self.imag
        return _tmp443
    #IF_CHECK_COMPLEX(T=t)
    if t in complextypes_set:
        #SUB_COMPLEX_COMPLEX(LHS=self; RHS=other)
        #RETURN_COMPLEX2(REAL=self.real - other.real; IMAG=self.imag - other.imag)
        _tmp464 = self.imag - other.imag
        if not _tmp464:
            return self.real - other.real
        #RETURN_COMPLEX(REAL=self.real - other.real; IMAG=_tmp464)
        _tmp471 = new(cls)
        _tmp471.real = self.real - other.real
        _tmp471.imag = _tmp464
        return _tmp471
    return NotImplemented

def complex_rsub(self, other, new=object.__new__, cls=mpqc, realtypes_set=realtypes_set):
    t = type(other)
    #IF_CHECK_REAL(T=t)
    if t in realtypes_set:
        #SUB_REAL_COMPLEX(LHS=other; RHS=self)
        #RETURN_COMPLEX(REAL=other - self.real; IMAG=-self.imag)
        _tmp492 = new(cls)
        _tmp492.real = other - self.real
        _tmp492.imag = -self.imag
        return _tmp492
    if t is complex:
        #SUB_COMPLEX_COMPLEX(LHS=other; RHS=self)
        #RETURN_COMPLEX2(REAL=other.real - self.real; IMAG=other.imag - self.imag)
        _tmp506 = other.imag - self.imag
        if not _tmp506:
            return other.real - self.real
        #RETURN_COMPLEX(REAL=other.real - self.real; IMAG=_tmp506)
        _tmp513 = new(cls)
        _tmp513.real = other.real - self.real
        _tmp513.imag = _tmp506
        return _tmp513
    return NotImplemented

def complex_mul(self, other, new=object.__new__, cls=mpqc, realtypes_set=realtypes_set, complextypes_set=complextypes_set):
    t = type(other)
    #IF_CHECK_COMPLEX(T=t)
    if t in complextypes_set:
        #MUL_COMPLEX_COMPLEX(LHS=self; RHS=other)
        a, b = self.real, self.imag
        c, d = other.real, other.imag
        #RETURN_COMPLEX2(REAL=a*c-b*d; IMAG=b*c+a*d)
        _tmp534 = b*c+a*d
        if not _tmp534:
            return a*c-b*d
        #RETURN_COMPLEX(REAL=a*c-b*d; IMAG=_tmp534)
        _tmp541 = new(cls)
        _tmp541.real = a*c-b*d
        _tmp541.imag = _tmp534
        return _tmp541
    #IF_CHECK_REAL(T=t)
    if t in realtypes_set:
        #MUL_COMPLEX_REAL(LHS=self; RHS=other)
        if not other:
            return 0
        #RETURN_COMPLEX(REAL=self.real*other; IMAG=self.imag*other)
        _tmp562 = new(cls)
        _tmp562.real = self.real*other
        _tmp562.imag = self.imag*other
        return _tmp562
    return NotImplemented

def complex_div(self, other, new=object.__new__, cls=mpqc, realtypes_set=realtypes_set, complextypes_set=complextypes_set):
    t = type(other)
    #IF_CHECK_REAL(T=t)
    if t in realtypes_set:
        #DIV_COMPLEX_REAL(LHS=self; RHS=other; MOD=%)
        #DIV_VALUE_VALUE(LHS=self.real; RHS=other; RESULT=re; MOD=%)
        _p, _q = self.real, other
        if not _q:
            raise ZeroDivisionError(repr(self.real) + " / " + repr(other))
        #IF_CHECK_INT(T=type(_p))
        if type(_p) in inttypes_set:
            #IF_CHECK_INT(T=type(_q))
            if type(_q) in inttypes_set:
                #FRACTION_NORMALIZE(NUMER=_p; DENOM=_q; RNUMER=_rp; RDENOM=_rq; MOD=%)
                _rp = _x = _p
                _rq = _y = _q
                while 1:
                    _x, _y = _y, _x % _y
                    if not _y:
                        break
                if _x != 1:
                    _rp //= _x
                    _rq //= _x
                re = _rp if _rq == 1 else mpq((_rp, _rq))
            else:
                re = _p / _q
        else:
            re = _p / _q
        #DIV_VALUE_VALUE(LHS=self.imag; RHS=other; RESULT=im; MOD=%)
        _p, _q = self.imag, other
        if not _q:
            raise ZeroDivisionError(repr(self.imag) + " / " + repr(other))
        #IF_CHECK_INT(T=type(_p))
        if type(_p) in inttypes_set:
            #IF_CHECK_INT(T=type(_q))
            if type(_q) in inttypes_set:
                #FRACTION_NORMALIZE(NUMER=_p; DENOM=_q; RNUMER=_rp; RDENOM=_rq; MOD=%)
                _rp = _x = _p
                _rq = _y = _q
                while 1:
                    _x, _y = _y, _x % _y
                    if not _y:
                        break
                if _x != 1:
                    _rp //= _x
                    _rq //= _x
                im = _rp if _rq == 1 else mpq((_rp, _rq))
            else:
                im = _p / _q
        else:
            im = _p / _q
        #RETURN_COMPLEX(REAL=re; IMAG=im)
        _tmp639 = new(cls)
        _tmp639.real = re
        _tmp639.imag = im
        return _tmp639
    #IF_CHECK_COMPLEX(T=t)
    if t in complextypes_set:
        #DIV_COMPLEX_COMPLEX(LHS=self; RHS=other; MOD=%)
        a, b = self.real, self.imag
        c, d = other.real, other.imag
        mag = c*c + d*d
        _tmp653 = b*c-a*d
        #DIV_VALUE_VALUE(LHS=a*c+b*d; RHS=mag; RESULT=re; MOD=%)
        _p, _q = a*c+b*d, mag
        if not _q:
            raise ZeroDivisionError(repr(a*c+b*d) + " / " + repr(mag))
        #IF_CHECK_INT(T=type(_p))
        if type(_p) in inttypes_set:
            #IF_CHECK_INT(T=type(_q))
            if type(_q) in inttypes_set:
                #FRACTION_NORMALIZE(NUMER=_p; DENOM=_q; RNUMER=_rp; RDENOM=_rq; MOD=%)
                _rp = _x = _p
                _rq = _y = _q
                while 1:
                    _x, _y = _y, _x % _y
                    if not _y:
                        break
                if _x != 1:
                    _rp //= _x
                    _rq //= _x
                re = _rp if _rq == 1 else mpq((_rp, _rq))
            else:
                re = _p / _q
        else:
            re = _p / _q
        if not _tmp653:
            return re
        #DIV_VALUE_VALUE(LHS=_tmp653; RHS=mag; RESULT=im; MOD=%)
        _p, _q = _tmp653, mag
        if not _q:
            raise ZeroDivisionError(repr(_tmp653) + " / " + repr(mag))
        #IF_CHECK_INT(T=type(_p))
        if type(_p) in inttypes_set:
            #IF_CHECK_INT(T=type(_q))
            if type(_q) in inttypes_set:
                #FRACTION_NORMALIZE(NUMER=_p; DENOM=_q; RNUMER=_rp; RDENOM=_rq; MOD=%)
                _rp = _x = _p
                _rq = _y = _q
                while 1:
                    _x, _y = _y, _x % _y
                    if not _y:
                        break
                if _x != 1:
                    _rp //= _x
                    _rq //= _x
                im = _rp if _rq == 1 else mpq((_rp, _rq))
            else:
                im = _p / _q
        else:
            im = _p / _q
        #RETURN_COMPLEX(REAL=re; IMAG=im)
        _tmp716 = new(cls)
        _tmp716.real = re
        _tmp716.imag = im
        return _tmp716
    return NotImplemented

def complex_rdiv(self, other, new=object.__new__, cls=mpqc, realtypes_set=realtypes_set):
    t = type(other)
    #IF_CHECK_REAL(T=t)
    if t in realtypes_set:
        #DIV_REAL_COMPLEX(LHS=other; RHS=self; MOD=%)
        _tmp730 = other
        c, d = self.real, self.imag
        mag = c*c + d*d
        #DIV_VALUE_VALUE(LHS=-_tmp730*d; RHS=mag; RESULT=im; MOD=%)
        _p, _q = -_tmp730*d, mag
        if not _q:
            raise ZeroDivisionError(repr(-_tmp730*d) + " / " + repr(mag))
        #IF_CHECK_INT(T=type(_p))
        if type(_p) in inttypes_set:
            #IF_CHECK_INT(T=type(_q))
            if type(_q) in inttypes_set:
                #FRACTION_NORMALIZE(NUMER=_p; DENOM=_q; RNUMER=_rp; RDENOM=_rq; MOD=%)
                _rp = _x = _p
                _rq = _y = _q
                while 1:
                    _x, _y = _y, _x % _y
                    if not _y:
                        break
                if _x != 1:
                    _rp //= _x
                    _rq //= _x
                im = _rp if _rq == 1 else mpq((_rp, _rq))
            else:
                im = _p / _q
        else:
            im = _p / _q
        #DIV_VALUE_VALUE(LHS= _tmp730*c; RHS=mag; RESULT=re; MOD=%)
        _p, _q = _tmp730*c, mag
        if not _q:
            raise ZeroDivisionError(repr(_tmp730*c) + " / " + repr(mag))
        #IF_CHECK_INT(T=type(_p))
        if type(_p) in inttypes_set:
            #IF_CHECK_INT(T=type(_q))
            if type(_q) in inttypes_set:
                #FRACTION_NORMALIZE(NUMER=_p; DENOM=_q; RNUMER=_rp; RDENOM=_rq; MOD=%)
                _rp = _x = _p
                _rq = _y = _q
                while 1:
                    _x, _y = _y, _x % _y
                    if not _y:
                        break
                if _x != 1:
                    _rp //= _x
                    _rq //= _x
                re = _rp if _rq == 1 else mpq((_rp, _rq))
            else:
                re = _p / _q
        else:
            re = _p / _q
        #RETURN_COMPLEX(REAL=re; IMAG=im)
        _tmp793 = new(cls)
        _tmp793.real = re
        _tmp793.imag = im
        return _tmp793
    if t is complex:
        #DIV_COMPLEX_COMPLEX(LHS=other; RHS=self; MOD=%)
        a, b = other.real, other.imag
        c, d = self.real, self.imag
        mag = c*c + d*d
        _tmp800 = b*c-a*d
        #DIV_VALUE_VALUE(LHS=a*c+b*d; RHS=mag; RESULT=re; MOD=%)
        _p, _q = a*c+b*d, mag
        if not _q:
            raise ZeroDivisionError(repr(a*c+b*d) + " / " + repr(mag))
        #IF_CHECK_INT(T=type(_p))
        if type(_p) in inttypes_set:
            #IF_CHECK_INT(T=type(_q))
            if type(_q) in inttypes_set:
                #FRACTION_NORMALIZE(NUMER=_p; DENOM=_q; RNUMER=_rp; RDENOM=_rq; MOD=%)
                _rp = _x = _p
                _rq = _y = _q
                while 1:
                    _x, _y = _y, _x % _y
                    if not _y:
                        break
                if _x != 1:
                    _rp //= _x
                    _rq //= _x
                re = _rp if _rq == 1 else mpq((_rp, _rq))
            else:
                re = _p / _q
        else:
            re = _p / _q
        if not _tmp800:
            return re
        #DIV_VALUE_VALUE(LHS=_tmp800; RHS=mag; RESULT=im; MOD=%)
        _p, _q = _tmp800, mag
        if not _q:
            raise ZeroDivisionError(repr(_tmp800) + " / " + repr(mag))
        #IF_CHECK_INT(T=type(_p))
        if type(_p) in inttypes_set:
            #IF_CHECK_INT(T=type(_q))
            if type(_q) in inttypes_set:
                #FRACTION_NORMALIZE(NUMER=_p; DENOM=_q; RNUMER=_rp; RDENOM=_rq; MOD=%)
                _rp = _x = _p
                _rq = _y = _q
                while 1:
                    _x, _y = _y, _x % _y
                    if not _y:
                        break
                if _x != 1:
                    _rp //= _x
                    _rq //= _x
                im = _rp if _rq == 1 else mpq((_rp, _rq))
            else:
                im = _p / _q
        else:
            im = _p / _q
        #RETURN_COMPLEX(REAL=re; IMAG=im)
        _tmp863 = new(cls)
        _tmp863.real = re
        _tmp863.imag = im
        return _tmp863
    return NotImplemented

def complex_pow(self, other, m=None, new=object.__new__, cls=mpqc, inttypes_set=inttypes_set):
    t = type(other)
    #IF_CHECK_INT(T=t)
    if t in inttypes_set:
        if not other:
            return 1
        if other==1:
            return self
        if other==2:
            return self*self
        if other < 0:
            base, exponent = 1/self, -other
        else:
            base, exponent = self, other
        #POW_COMPLEX_INT(LHS=base; RHS=exponent; MOD=%)
        a, b = base.real, base.imag
        n = exponent
        if not a:
            case = n % 4
            if not case:
                return b**n
            elif case == 1:
                #RETURN_COMPLEX(REAL=0; IMAG=b**n)
                _tmp884 = new(cls)
                _tmp884.real = 0
                _tmp884.imag = b**n
                return _tmp884
            elif case == 2:
                return -(b**n)
            else:
                #RETURN_COMPLEX(REAL=0; IMAG=-b**n)
                _tmp891 = new(cls)
                _tmp891.real = 0
                _tmp891.imag = -b**n
                return _tmp891
        ta, tb = type(a), type(b)
        m = 1
        if ta is mpq:
            if tb is mpq:
                m = (a[1] * b[1]) ** n
                a, b = a[0]*b[1], a[1]*b[0]
            #ELIF_CHECK_INT(T=tb)
            elif tb in inttypes_set:
                m = a[1] ** n
                a, b = a[0], a[1]*b
        elif tb is mpq:
            #IF_CHECK_INT(T=ta)
            if ta in inttypes_set:
                m = b[1] ** n
                a, b = a*b[1], b[0]
        c, d = 1, 0
        while 1:
            if n & 1:
                c, d = a*c-b*d, b*c+a*d
                n -= 1
                if not n:
                    break
            a, b = a*a-b*b, 2*a*b
            n //= 2
        if m==1:
            #RETURN_COMPLEX2(REAL=c; IMAG=d)
            _tmp912 = d
            if not _tmp912:
                return c
            #RETURN_COMPLEX(REAL=c; IMAG=_tmp912)
            _tmp919 = new(cls)
            _tmp919.real = c
            _tmp919.imag = _tmp912
            return _tmp919
        # c,d,m are integers
        #FRACTION_NORMALIZE(NUMER=c; DENOM=m; RNUMER=re_p; RDENOM=re_q; MOD=%)
        re_p = _x = c
        re_q = _y = m
        while 1:
            _x, _y = _y, _x % _y
            if not _y:
                break
        if _x != 1:
            re_p //= _x
            re_q //= _x
        re = re_p if re_q==1 else mpq((re_p, re_q))
        if not d:
            return re
        #FRACTION_NORMALIZE(NUMER=d; DENOM=m; RNUMER=im_p; RDENOM=im_q; MOD=%)
        im_p = _x = d
        im_q = _y = m
        while 1:
            _x, _y = _y, _x % _y
            if not _y:
                break
        if _x != 1:
            im_p //= _x
            im_q //= _x
        if im_q==1:
            im = im_p
        else:
            im = mpq((im_p, im_q))
        #RETURN_COMPLEX(REAL=re; IMAG=im)
        _tmp940 = new(cls)
        _tmp940.real = re
        _tmp940.imag = im
        return _tmp940
    return NotImplemented
    
def fraction_lt(self, other, cls=mpq, inttypes_set=inttypes_set):
    p, q = self
    t = type(other)
    #IF_CHECK_INT(T=t)
    if t in inttypes_set:
        return (p < q*other)
    elif t is cls:
        r, s = other
        return p*s < q*r
    return NotImplemented
    
def fraction_le(self, other, cls=mpq, inttypes_set=inttypes_set):
    p, q = self
    t = type(other)
    #IF_CHECK_INT(T=t)
    if t in inttypes_set:
        return (p <= q*other)
    elif t is cls:
        r, s = other
        return p*s <= q*r
    return NotImplemented
    
def fraction_gt(self, other, cls=mpq, inttypes_set=inttypes_set):
    p, q = self
    t = type(other)
    #IF_CHECK_INT(T=t)
    if t in inttypes_set:
        return (p > q*other)
    elif t is cls:
        r, s = other
        return p*s > q*r
    return NotImplemented
    
def fraction_ge(self, other, cls=mpq, inttypes_set=inttypes_set):
    p, q = self
    t = type(other)
    #IF_CHECK_INT(T=t)
    if t in inttypes_set:
        return (p >= q*other)
    elif t is cls:
        r, s = other
        return p*s >= q*r
    return NotImplemented
    



"""Provides classes BasicAlgebra, Verbatim, and CollectingField.
"""
__docformat__ = 'restructuredtext'

from ..core import init_module
from .algebra import Algebra, SymbolicEquality
from .verbatim import Verbatim
#from .pairs import CollectingField
#from ..ring import CommutativeRing as CollectingField

    



"""
Simplified functions for using the Fred API.
"""

import os
from .core import Fred


def key(api_key):
    os.environ['FRED_API_KEY'] = api_key
    return Fred()


#####################
# Category
#####################

def category(**kwargs):
    """Get a category."""
    if 'series' in kwargs:
        kwargs.pop('series')
        path = 'series'
    else:
        path = None
    return Fred().category(path, **kwargs)


def categories(identifier, **kwargs):
    """Just in case someone misspells the method."""
    kwargs['category_id'] = identifier
    return category(**kwargs)


def children(**kwargs):
    """Get child categories for a specified parent category."""
    return Fred().category('children', **kwargs)


def related(identifier, **kwargs):
    """Get related categories for a specified category."""
    kwargs['category_id'] = identifier
    return Fred().category('related', **kwargs)


def category_series(identifier, **kwargs):
    """Get the series in a category."""
    kwargs['category_id'] = identifier
    return Fred().category('series', **kwargs)


#####################
# Releases
#####################

def release(release_id, **kwargs):
    """Get the release of economic data."""
    kwargs['release_id'] = release_id
    return Fred().release(**kwargs)


def releases(release_id=None, **kwargs):
    """Get all releases of economic data."""
    if not 'id' in kwargs and release_id is not None:
        kwargs['release_id'] = release_id
        return Fred().release(**kwargs)
    return Fred().releases(**kwargs)


def dates(**kwargs):
    """Get release dates for economic data."""
    return Fred().releases('dates', **kwargs)


#####################
# Series
#####################

def series(identifier=None, **kwargs):
    """Get an economic data series."""
    if identifier:
        kwargs['series_id'] = identifier
    if 'release' in kwargs:
        kwargs.pop('release')
        path = 'release'
    elif 'releases' in kwargs:
        kwargs.pop('releases')
        path = 'release'
    else:
        path = None
    return Fred().series(path, **kwargs)


def observations(identifier, **kwargs):
    """Get an economic data series."""
    kwargs['series_id'] = identifier
    return Fred().series('observations', **kwargs)


def search(text, **kwargs):
    """Get economic data series that match keywords."""
    kwargs['search_text'] = text
    return Fred().series('search', **kwargs)


def updates(**kwargs):
    """Get economic data series sorted in descending order."""
    return Fred().series('updates', **kwargs)


def vintage(identifier, **kwargs):
    """
    Get the dates in history when a series' data values were revised or new
    data values were released.
    """
    kwargs['series_id'] = identifier
    return Fred().series('vintagedates', **kwargs)


#####################
# Sources
#####################

def source(source_id=None, **kwargs):
    """Get a source of economic data."""
    if source_id is not None:
        kwargs['source_id'] = source_id
    elif 'id' in kwargs:
        source_id = kwargs.pop('id')
        kwargs['source_id'] = source_id
    if 'releases' in kwargs:
        kwargs.pop('releases')
        path = 'releases'
    else:
        path = None
    return Fred().source(path, **kwargs)


def sources(source_id=None, **kwargs):
    """Get the sources of economic data."""
    if source_id or 'id' in kwargs:
        return source(source_id, **kwargs)
    return Fred().sources(**kwargs)




__all__ = ['SYMBOL']

import re

from ..core import init_module
init_module.import_heads()
init_module.import_numbers()
init_module.import_lowlevel_operations()

from .base import AtomicHead, heads_precedence, Expr, Pair, ArithmeticHead, NotImplementedHeadMethod

_is_atomic = re.compile(r'\A\w+\Z').match

class SymbolHead(AtomicHead):
    """
    SymbolHead is a head for symbols, data can be any Python object.
    """

    def new(self, cls, data, evaluate=True):
        return cls(self, data)

    def reevaluate(self, cls, data):
        return cls(self, data)

    def is_data_ok(self, cls, data):
        return

    def __repr__(self): return 'SYMBOL'

    def data_to_str_and_precedence(self, cls, data):
        if isinstance(data, Expr):
            h, d = data.pair
            return h.data_to_str_and_precedence(cls, d)
        s = str(data)
        if _is_atomic(s):
            return s, heads_precedence.SYMBOL
        return s, 0.0 # force parenthesis

    def to_EXP_COEFF_DICT(self, cls, data, expr, variables = None):
        variables = EXP_COEFF_DICT.combine_variables(data, variables)
        exp = EXP_COEFF_DICT.make_exponent(data, variables)
        assert len(exp)==len(variables), `exp, variables, i, data`
        return cls(EXP_COEFF_DICT, Pair(variables, {exp:1}))

    def commutative_mul_number(self, cls, lhs, rhs):
        return term_coeff_new(cls, (lhs, rhs))

    non_commutative_mul_number = commutative_mul_number

    def pow(self, cls, base, exp):
        if type(exp) is cls and exp.head is NUMBER:
            exp = exp.data
        return pow_new(cls, (base, exp))

    def pow_number(self, cls, base, exp):
        if exp==1: return base
        if exp==0: return cls(NUMBER, 1)
        return cls(POW, (base, exp))

    def expand(self, cls, expr):
        return expr

    def expand_intpow(self, cls, expr, intexp):
        if intexp==0:
            return cls(NUMBER, 1)
        return cls(POW, (expr, intexp))

    def diff(self, cls, data, expr, symbol, order, cache={}):
        if order==0:
            return expr
        if data == symbol:
            assert order>0,`order`
            return cls(NUMBER, int(order==1))
        return cls(NUMBER, 0)

    def fdiff(self, cls, data, expr, argument_index, order):
        vcls = cls.get_value_algebra()
        dcls = cls.get_differential_algebra()
        d = dcls(FDIFF, vcls(NUMBER, argument_index))**order
        return cls(APPLY, (d, (expr,)))

    def apply(self, cls, data, func, args):
        return cls(APPLY, (func, args))

    def integrate_indefinite(self, cls, data, expr, x):
        if data==x:
            return cls(TERM_COEFF, (cls(POW, (expr, 2)), mpq((1,2))))
        return cls(BASE_EXP_DICT, {expr:1, cls(SYMBOL, x):1})

    def integrate_definite(self, cls, data, expr, x, a, b):
        if data==x:
            return (b**2-a**2)/2
        return expr*(b-a)

    def to_TERM_COEFF_DICT(self, Algebra, data, expr):
        return expr

    def to_MUL(self, Algebra, data, expr):
        return expr

    def to_ADD(self, cls, data, expr):
        return expr

    def algebra_pos(self, Algebra, expr):
        return expr

    def algebra_neg(self, Algebra, expr):
        if Algebra.algebra_options.get('is_additive_group_commutative'):
            if Algebra.algebra_options.get('evaluate_addition'):
                return Algebra(TERM_COEFF, (expr, -1))
        return Algebra(NEG, expr)

    def algebra_add_number(self, Algebra, lhs, rhs, inplace):
        if Algebra.algebra_options.get('evaluate_addition'):
            if not rhs:
                return lhs            
            if Algebra.algebra_options.get('is_additive_group_commutative'):
                return Algebra(TERM_COEFF_DICT, {Algebra(NUMBER, 1): rhs, lhs:1})
        return Algebra(ADD, [lhs, Algebra(NUMBER, rhs)])

    def algebra_add(self, Algebra, lhs, rhs, inplace):
        rhead, rdata = rhs.pair
        if Algebra.algebra_options.get('is_additive_group_commutative'):
            ldata = lhs.data
            if Algebra.algebra_options.get('evaluate_addition'):
                if rhead is ADD or rhead is EXP_COEFF_DICT or rhead is MUL or rhead is NEG:
                    rhead, rdata = rhs.to(TERM_COEFF_DICT).pair
                if rhead is SYMBOL:
                    if ldata == rdata:
                        return Algebra(TERM_COEFF, (lhs, 2))
                    return Algebra(TERM_COEFF_DICT, {lhs: 1, rhs: 1})
                if rhead is NUMBER:
                    if rdata:
                        return Algebra(TERM_COEFF_DICT, {Algebra(NUMBER, 1): rdata, lhs:1})
                    return lhs
                if rhead is TERM_COEFF:
                    term, coeff = rdata
                    if term==lhs:
                        return term_coeff_new(Algebra, (term, coeff+1))
                    return Algebra(TERM_COEFF_DICT, {term: coeff, lhs:1})
                if rhead is TERM_COEFF_DICT:
                    d = rdata.copy()
                    term_coeff_dict_add_item(Algebra, d, lhs, 1)
                    return term_coeff_dict_new(Algebra, d)
                if rhead is POW or rhead is BASE_EXP_DICT:
                    return Algebra(TERM_COEFF_DICT, {lhs:1, rhs:1})
            else:
                if rhead is TERM_COEFF_DICT or rhead is EXP_COEFF_DICT:
                    rhs = rhs.to(ADD)
                    rhead, rdata = rhs.pair
                if rhead is ADD:
                    data = [lhs] + rdata
                else:
                    data = [lhs, rhs]
                return add_new(Algebra, [lhs, rhs])
            return super(type(self), self).algebra_add(Algebra, lhs, rhs, inplace)
        else:
            if rhead is TERM_COEFF_DICT or rhead is EXP_COEFF_DICT:
                rhs = rhs,to(ADD)
                rhead, rdata = rhs.pair
            if rhead is ADD:
                data = [lhs] + rdata
            else:
                data = [lhs, rhs]
            if Algebra.algebra_options.get('evaluate_addition'):
                ADD.combine_add_list(Algebra, data)
            return add_new(Algebra, data)

    def algebra_mul_number(self, Algebra, lhs, rhs, inplace):
        if Algebra.algebra_options.get('evaluate_addition'):
            return term_coeff_new(Algebra, (lhs, rhs))
        return mul_new(Algebra, [lhs, Algebra(NUMBER, rhs)])

    def algebra_mul(self, Algebra, lhs, rhs, inplace):
        if Algebra.algebra_options.get('is_additive_group'):
            if Algebra.algebra_options.get('evaluate_addition'):
                rhead, rdata = rhs.pair
                if rhead is NUMBER:
                    return term_coeff_new(Algebra, (lhs, rdata))
            else:
                return mul_new(Algebra, [lhs, rhs])
        if Algebra.algebra_options.get('is_multiplicative_group'):
            if Algebra.algebra_options.get('evaluate_multiplication'):
                rhead, rdata = rhs.pair
                if Algebra.algebra_options.get('is_multiplicative_group_commutative'):
                    if rhead is MUL or EXP_COEFF_DICT:
                        rhs = rhead.to_BASE_EXP_DICT(Algebra, rdata, rhs)
                        rhead, rdata = rhs.pair
                    if rhead is NUMBER:
                        return term_coeff_new(Algebra, (lhs, rdata))
                    if rhead is BASE_EXP_DICT:
                        data = rdata.copy()
                        base_exp_dict_add_item(Algebra, data, lhs, 1)
                        return base_exp_dict_new(Algebra, data)
                    if lhs==rhs:
                        return Algebra(POW, (lhs, 2))
                    return Algebra(BASE_EXP_DICT, {lhs:1, rhs:1})
                else:
                    if rhead is MUL:
                        data = [lhs] + rdata
                    else:
                        data = [lhs, rhs]
                    coeff = MUL.combine_mul_list(Algebra, data)
                    assert coeff is 1,`coeff`
                    return mul_new(Algebra, data)
            else:
                pass
        return super(type(self), self).algebra_mul(Algebra, lhs, rhs, inplace)

    def algebra_pow_number(self, Algebra, lhs, rhs, inplace):
        if Algebra.algebra_options.get('is_multiplicative_group'):
            if Algebra.algebra_options.get('evaluate_multiplication'):
                if Algebra.algebra_options.get('is_multiplicative_group_commutative'):
                    return pow_new(Algebra, (lhs, rhs))
                else:
                    return pow_new(Algebra, (lhs, rhs))
            else:
                return cls(POW, (lhs, rhs))
        return super(type(self), self).algebra_pow_number(Algebra, lhs, rhs, inplace)

SYMBOL = SymbolHead()



#!/usr/bin/env python
# Parallel Python setup script
# For the latest version of the Parallel Python
# software visit: http://www.parallelpython.com
"""
Standard build tool for python libraries.
"""

import os.path
from distutils.core import setup

from pp import version as VERSION


LONG_DESCRIPTION = """
Parallel Python module (PP) provides an easy and efficient way to create \
parallel-enabled applications for SMP computers and clusters. PP module \
features cross-platform portability and dynamic load balancing. Thus \
application written with PP will parallelize efficiently even on \
heterogeneous and multi-platform clusters (including clusters running other \
application with variable CPU loads). Visit http://www.parallelpython.com \
for further information.
"""


setup(
        name="pp",
        url="http://www.parallelpython.com",
        version=VERSION,
        download_url="http://www.parallelpython.com/downloads/pp/pp-%s.zip" % (
            VERSION),
        author="Vitalii Vanovschi",
        author_email="support@parallelpython.com",
        py_modules=["pp", "ppauto", "ppcommon", "pptransport", "ppworker"],
        scripts=["ppserver.py"],
        description="Parallel and distributed programming for Python",
        platforms=["Windows", "Linux", "Unix"],
        long_description=LONG_DESCRIPTION,
        license="BSD-like",
        classifiers=[
        "Topic :: Software Development",
        "Topic :: System :: Distributed Computing",
        "Programming Language :: Python",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: BSD License",
        "Natural Language :: English",
        "Intended Audience :: Developers",
        "Development Status :: 5 - Production/Stable",
        ],
)



""" Implementes functions ring support.
"""
#
# Author: Pearu Peterson
# Created: April, 2008
#

__all__ = ['FunctionRing']

from ..core import classes, objects, init_module
from ..basealgebra import Verbatim, Algebra
from ..ring import CommutativeRing

init_module.import_heads()


class FunctionRing(CommutativeRing):
    """ Base class to functions ring classes.

    Use ``Function`` function to construct instances.
    """

    argument_algebras = None
    nargs = None

    @classmethod
    def get_value_algebra(cls):
        return CommutativeRing

    def get_argument_algebra(self, index):
        return self.get_value_algebra()

    @classmethod
    def get_function_algebra(cls):
        return classes.OperatorRing

    @classmethod
    def get_differential_algebra(cls):
        return classes.DifferentialRing

    @classmethod
    def get_predefined_symbols(cls, name):
        if name=='D': return D
        return

    @classmethod
    def convert(cls, obj, typeerror=True):
        tobj = type(obj)
        if tobj is cls:
            return obj        
        if isinstance(obj, cls.get_value_algebra()):
            return cls(NUMBER, obj)
        return super(CommutativeRing, cls).convert(obj, typeerror=typeerror)

    def as_algebra(self, cls, typeerror=True):
        if cls is classes.Verbatim:
            return self.as_verbatim()
        if type(self) is cls:
            return self
        #if isinstance(self, cls):
        #    return self.as_verbatim().as_algebra(cls)
        if typeerror:
            raise TypeError('Cannot convert %s to %s instance' % (type(self).__name__, cls.__name__))
        return NotImplemented

    def __call__(self, *args, **options):
        cls = self.get_value_algebra()
        #cls = classes.Calculus
        evaluate = options.get('evaluate', True)
        if evaluate:
            result = self.head.apply(cls, self.data, self, args)
            if result is not NotImplemented:
                return result
        return cls(APPLY, (self, args))

classes.FunctionRing = FunctionRing





__all__ = ['BASE_EXP_DICT']

from .base import heads, heads_precedence, ArithmeticHead

from ..core import init_module, Expr
init_module.import_heads()
init_module.import_numbers()
init_module.import_lowlevel_operations()

class BaseExpDictHead(ArithmeticHead):
    """ BASE_EXP_DICT expression data is a dictionary of base and
    exponent pairs. All base parts must be Expr instances.

    For example, ``Algebra(BASE_EXP_DICT. {x:2, y:a, 3:1, 2:1/2})``
    represents ``3 * 2**(1/2) * x**2 * y**a``.
    """

    def is_data_ok(self, cls, data):
        if type(data) is dict:
            for item in data.iteritems():
                msg = POW.is_data_ok(cls, item)
                if msg:
                    return 'POW data=%s: %s' % (item, msg)
                b, e = item
                if b.head is POW:
                    return 'BASE_EXP_DICT key cannot be POW'
        else:
            return 'data must be dict instance but got %s' % (type(data))
        return


    def __repr__(self): return 'BASE_EXP_DICT'

    def data_to_str_and_precedence(self, cls, base_exp_dict):
        factors = []
        coeff = None
        for base, exp in base_exp_dict.items():
            if exp==1 and base.head is NUMBER:
                coeff = base.data
            else:
                factors.append(cls(POW, (base, exp)))
        if coeff is not None:
            return TERM_COEFF.data_to_str_and_precedence(cls, (cls(MUL, factors), coeff))
        return MUL.data_to_str_and_precedence(cls, factors)

    def reevaluate(self, cls, data):
        r = cls(NUMBER, 1)
        for base, exp in data.iteritems():
            r *= base ** exp
        return r

    def to_ADD(self, Algebra, base_exp_dict, expr):
        return Algebra(ADD, [expr])

    def term_coeff(self, cls, expr):
        data = expr.data
        coeff = base_exp_dict_get_coefficient(cls, data)
        if coeff is not None:
            data = data.copy()
            del data[coeff]
            r = base_exp_dict_new(cls, data)
            t, c = r.head.term_coeff(cls, r)
            return t, c * coeff 
        return expr, 1

    def new(self, cls, base_exp_dict, evaluate=True):
        return base_exp_dict_new(cls, base_exp_dict)

    def neg(self, cls, expr):
        data = expr.data
        coeff = base_exp_dict_get_coefficient(cls, data)
        if coeff is None:
            return cls(TERM_COEFF, (expr, -1))
        data = data.copy()
        del data[coeff]
        return term_coeff_new(cls, (base_exp_dict_new(cls, data), -coeff))

    def inplace_commutative_data_mul(self, cls, data, rhs):
        """
        Multiply base-exp-dictionary with rhs inplace.
        """
        rhead, rdata = rhs.pair
        if rhead is SYMBOL or rhead is ADD or rhead is APPLY or rhead is DIFF or rhead is FDIFF:
            base_exp_dict_add_item(cls, data, rhs, 1)
        elif rhead is NUMBER:
            base_exp_dict_add_item(cls, data, rhs, 1)
        elif rhead is TERM_COEFF:
            term, coeff = rdata
            base_exp_dict_add_item(cls, data, term, 1)
            base_exp_dict_add_item(cls, data, cls(NUMBER, coeff), 1)
        elif rhead is BASE_EXP_DICT:
            base_exp_dict_add_dict(cls, data, rdata)
        elif rhead is POW:
            base, exp = rdata
            base_exp_dict_add_item(cls, data, base, exp)
        elif rhead is TERM_COEFF_DICT:
            base_exp_dict_add_item(cls, data, rhs, 1)
        else:
            raise NotImplementedError(`self, cls, rhs.pair`)
    
    def commutative_mul(self, cls, lhs, rhs):
        data = lhs.data.copy()
        self.inplace_commutative_data_mul(cls, data, rhs)
        return base_exp_dict_new(cls, data)

    def commutative_mul_number(self, cls, lhs, rhs):
        return term_coeff_new(cls, (lhs, rhs))

    def commutative_div_number(self, cls, lhs, rhs):
        r = number_div(cls, 1, rhs)
        if rhs==0:
            return r * lhs
        return term_coeff_new(cls, (lhs, r))

    def commutative_div(self, cls, lhs, rhs):
        rhead, rdata = rhs.pair
        if rhead is NUMBER:
            return self.commutative_div_number(cls, lhs, rdata)
        if rhead is POW:
            data = lhs.data.copy()
            base, exp = rdata
            base_exp_dict_sub_item(cls, data, base, exp)
            return base_exp_dict_new(cls, data)
        if rhead is BASE_EXP_DICT:
            data = lhs.data.copy()
            base_exp_dict_sub_dict(cls, data, rdata)
            return base_exp_dict_new(cls, data)
        if rhead is SYMBOL or rhead is TERM_COEFF_DICT or rhead is APPLY:
            data = lhs.data.copy()
            base_exp_dict_sub_item(cls, data, rhs, 1)
            return base_exp_dict_new(cls, data)
        if rhead is TERM_COEFF:
            term, coeff = rhs.term_coeff()
            return (lhs / term) / coeff
        return ArithmeticHead.commutative_div(self, cls, lhs, rhs)

    def commutative_rdiv_number(self, cls, lhs, rhs):
        data = lhs.data.copy()
        base_exp_dict_mul_value(cls, data, -1)
        return base_exp_dict_new(cls, data) * rhs
    
    def scan(self, proc, cls, data, target):
        for b, e in data.iteritems():
            b.head.scan(proc, cls, b.data, target)
            if isinstance(e, Expr):
                e.head.scan(proc, cls, e.data, target)
            else:
                NUMBER.scan(proc, cls, e, target)
        proc(cls, self, data, target)

    def walk(self, func, cls, data, target):
        d = {}
        flag = False
        for b, e in data.iteritems():
            b1 = b.head.walk(func, cls, b.data, b)
            if isinstance(e, Expr):
                e1 = e.head.walk(func, cls, e.data, e)
            else:
                e1 = NUMBER.walk(func, cls, e, e)
            if b1 is not b or e1 is not e:
                flag = True
            self.inplace_commutative_data_mul(cls, d, b1**e1)
        if flag:
            r = base_exp_dict_new(cls, d)
            return func(cls, r.head, r.data, r)
        return func(cls, self, data, target)

    def pow(self, cls, base, exp):
        if type(exp) is cls:
            h, d = exp.pair
            if h is NUMBER and isinstance(d, numbertypes):
                exp = d
        if isinstance(exp, inttypes):
            if exp:
                data = base.data.copy()
                base_exp_dict_mul_value(cls, data, exp)
                return base_exp_dict_new(cls, data)
            return cls(NUMBER, 1)
        return pow_new(cls, (base, exp))

    pow_number = pow

    def expand(self, cls, expr):
        data = {}
        for b, e in expr.data.items():
            f = pow_new(cls, (b, e)).expand()
            h, d = f.pair
            data1 = {}
            if h is TERM_COEFF_DICT:
                data2 = d
            else:
                t, c = f.term_coeff()
                data2 = {t: c}
            if data:
                term_coeff_dict_mul_dict(cls, data1, data, data2)
                data = data1
            else:
                data = data2
        return term_coeff_dict_new(cls, data)

    def diff(self, cls, data, expr, symbol, order, cache={}):
        key = (expr, symbol, order)
        result = cache.get(key)
        if result is not None:
            return result
        key1 = (expr, symbol, 1)
        result = cache.get(key1)
        if result is None:
            operands = data.items()
            zero = cls(NUMBER, 0)
            result = zero
            for i in range(len(operands)):
                p = pow_new(cls, operands[i])
                d = p.head.diff(cls, p.data, p, symbol, 1, cache=cache)
                if d==zero:
                    continue
                be_dict = data.copy()
                del be_dict[operands[i][0]]
                r = base_exp_dict_new(cls, be_dict)
                result += r * d
            cache[key1] = result
        if order>1:
            result = result.head.diff(cls, result.data, result, symbol, order-1, cache=cache)
            cache[key] = result
        return result

    def apply(self, cls, data, func, args):
        result = cls(NUMBER, 1)
        for base, exp in data.iteritems():
            if isinstance(exp, Expr):
                return NotImplemented
            result *= base.head.apply(cls, base.data, base, args) ** exp
        return result

    def integrate_indefinite(self, cls, data, expr, x):
        d1 = {} # f(x)**g(x)
        d2 = {} # f(x)**const
        d3 = {} # const**g(x)
        d4 = {} # const**const
        for base, exp in data.iteritems():
            if x in base.symbols_data:
                if type(exp) is cls and x in exp.symbols_data:
                    d1[base] = exp
                else:
                    d2[base] = exp                    
            elif type(exp) is cls and x in exp.symbols_data:
                d3[base] = exp
            else:
                d4[base] = exp
        if d1 or (d2 and d3) or (len(d2)>1) or (len(d3)>1):
            raise NotImplementedError("don't know how to integrate %s over %s" % (expr, x))
        if not (d2 or d3):
            return expr * cls(SYMBOL, x)            
        if d4:
            if len(d4)>1:
                const = cls(BASE_EXP_DICT, d4)
            else:
                const = pow_new(cls, dict_get_item(d4))
        else:
            const = 1
        if d2:
            newexpr = pow_new(cls, dict_get_item(d2))
            return newexpr.head.integrate_indefinite(cls, newexpr.data, newexpr, x) * const
        if d3:
            newexpr = pow_new(cls, dict_get_item(d3))
            return newexpr.head.integrate_indefinite(cls, newexpr.data, newexpr, x) * const
        raise NotImplementedError("don't know how to integrate %s over %s" % (expr, x))

    def integrate_definite(self, cls, data, expr, x, a, b):
        d1 = {} # f(x)**g(x)
        d2 = {} # f(x)**const
        d3 = {} # const**g(x)
        d4 = {} # const**const
        for base, exp in data.iteritems():
            if x in base.symbols_data:
                if type(exp) is cls and x in exp.symbols_data:
                    d1[base] = exp
                else:
                    d2[base] = exp                    
            elif type(exp) is cls and x in exp.symbols_data:
                d3[base] = exp
            else:
                d4[base] = exp
        if d1 or (d2 and d3) or (len(d2)>1) or (len(d3)>1):
            raise NotImplementedError("don't know how to integrate %s over %s in [%s, %s]" % (expr, x, a, b))
        if not (d2 or d3):
            return (b-a) * cls(SYMBOL, x)            
        if d4:
            if len(d4)>1:
                const = cls(BASE_EXP_DICT, d4)
            else:
                const = pow_new(cls, dict_get_item(d4))
        else:
            const = 1
        if d2:
            newexpr = pow_new(cls, dict_get_item(d2))
            return newexpr.head.integrate_definite(cls, newexpr.data, newexpr, x, a, b) * const
        if d3:
            newexpr = pow_new(cls, dict_get_item(d3))
            return newexpr.head.integrate_definite(cls, newexpr.data, newexpr, x, a, b) * const
        raise NotImplementedError("don't know how to integrate %s over %s in [%s, %s]" % (expr, x, a, b))
    
BASE_EXP_DICT = BaseExpDictHead()

